@@ -1,15 +1,21 @@
 """The Airly integration."""
+from __future__ import annotations
+
 from datetime import timedelta
 import logging
 from math import ceil
 
+from aiohttp import ClientSession
 from aiohttp.client_exceptions import ClientConnectorError
 from airly import Airly
 from airly.exceptions import AirlyError
 import async_timeout
 
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_API_KEY, CONF_LATITUDE, CONF_LONGITUDE
+from homeassistant.core import HomeAssistant
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
+from homeassistant.helpers.device_registry import async_get_registry
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator, UpdateFailed
 from homeassistant.util import dt as dt_util
 
@@ -30,7 +36,7 @@ PLATFORMS = ["air_quality", "sensor"]
 _LOGGER = logging.getLogger(__name__)
 
 
-def set_update_interval(instances, requests_remaining):
+def set_update_interval(instances_count: int, requests_remaining: int) -> timedelta:
     """
     Return data update interval.
 
@@ -46,7 +52,7 @@ def set_update_interval(instances, requests_remaining):
     interval = timedelta(
         minutes=min(
             max(
-                ceil(minutes_to_midnight / requests_remaining * instances),
+                ceil(minutes_to_midnight / requests_remaining * instances_count),
                 MIN_UPDATE_INTERVAL,
             ),
             MAX_UPDATE_INTERVAL,
@@ -58,19 +64,39 @@ def set_update_interval(instances, requests_remaining):
     return interval
 
 
-async def async_setup_entry(hass, config_entry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Airly as config entry."""
-    api_key = config_entry.data[CONF_API_KEY]
-    latitude = config_entry.data[CONF_LATITUDE]
-    longitude = config_entry.data[CONF_LONGITUDE]
-    use_nearest = config_entry.data.get(CONF_USE_NEAREST, False)
+    api_key = entry.data[CONF_API_KEY]
+    latitude = entry.data[CONF_LATITUDE]
+    longitude = entry.data[CONF_LONGITUDE]
+    use_nearest = entry.data.get(CONF_USE_NEAREST, False)
 
     # For backwards compat, set unique ID
-    if config_entry.unique_id is None:
+    if entry.unique_id is None:
         hass.config_entries.async_update_entry(
-            config_entry, unique_id=f"{latitude}-{longitude}"
+            entry, unique_id=f"{latitude}-{longitude}"
         )
 
+    # identifiers in device_info should use tuple[str, str] type, but latitude and
+    # longitude are float, so we convert old device entries to use correct types
+    # We used to use a str 3-tuple here sometime, convert that to a 2-tuple too.
+    device_registry = await async_get_registry(hass)
+    old_ids = (DOMAIN, latitude, longitude)
+    for old_ids in (
+        (DOMAIN, latitude, longitude),
+        (
+            DOMAIN,
+            str(latitude),
+            str(longitude),
+        ),
+    ):
+        device_entry = device_registry.async_get_device({old_ids})  # type: ignore[arg-type]
+        if device_entry and entry.entry_id in device_entry.config_entries:
+            new_ids = (DOMAIN, f"{latitude}-{longitude}")
+            device_registry.async_update_device(
+                device_entry.id, new_identifiers={new_ids}
+            )
+
     websession = async_get_clientsession(hass)
 
     update_interval = timedelta(minutes=MIN_UPDATE_INTERVAL)
@@ -81,21 +107,19 @@ async def async_setup_entry(hass, config_entry):
     await coordinator.async_config_entry_first_refresh()
 
     hass.data.setdefault(DOMAIN, {})
-    hass.data[DOMAIN][config_entry.entry_id] = coordinator
+    hass.data[DOMAIN][entry.entry_id] = coordinator
 
-    hass.config_entries.async_setup_platforms(config_entry, PLATFORMS)
+    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
 
     return True
 
 
-async def async_unload_entry(hass, config_entry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
-    unload_ok = await hass.config_entries.async_unload_platforms(
-        config_entry, PLATFORMS
-    )
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
 
     if unload_ok:
-        hass.data[DOMAIN].pop(config_entry.entry_id)
+        hass.data[DOMAIN].pop(entry.entry_id)
 
     return unload_ok
 
@@ -105,14 +129,14 @@ class AirlyDataUpdateCoordinator(DataUpdateCoordinator):
 
     def __init__(
         self,
-        hass,
-        session,
-        api_key,
-        latitude,
-        longitude,
-        update_interval,
-        use_nearest,
-    ):
+        hass: HomeAssistant,
+        session: ClientSession,
+        api_key: str,
+        latitude: float,
+        longitude: float,
+        update_interval: timedelta,
+        use_nearest: bool,
+    ) -> None:
         """Initialize."""
         self.latitude = latitude
         self.longitude = longitude
@@ -121,9 +145,9 @@ class AirlyDataUpdateCoordinator(DataUpdateCoordinator):
 
         super().__init__(hass, _LOGGER, name=DOMAIN, update_interval=update_interval)
 
-    async def _async_update_data(self):
+    async def _async_update_data(self) -> dict[str, str | float | int]:
         """Update data via library."""
-        data = {}
+        data: dict[str, str | float | int] = {}
         if self.use_nearest:
             measurements = self.airly.create_measurements_session_nearest(
                 self.latitude, self.longitude, max_distance_km=5
