@@ -1,6 +1,8 @@
 """Provide the device automations for Alarm control panel."""
 from __future__ import annotations
 
+from typing import Final
+
 import voluptuous as vol
 
 from homeassistant.components.alarm_control_panel.const import (
@@ -39,7 +41,7 @@ from .const import (
     CONDITION_TRIGGERED,
 )
 
-CONDITION_TYPES = {
+CONDITION_TYPES: Final[set[str]] = {
     CONDITION_TRIGGERED,
     CONDITION_DISARMED,
     CONDITION_ARMED_HOME,
@@ -48,7 +50,7 @@ CONDITION_TYPES = {
     CONDITION_ARMED_CUSTOM_BYPASS,
 }
 
-CONDITION_SCHEMA = DEVICE_CONDITION_BASE_SCHEMA.extend(
+CONDITION_SCHEMA: Final = DEVICE_CONDITION_BASE_SCHEMA.extend(
     {
         vol.Required(CONF_ENTITY_ID): cv.entity_id,
         vol.Required(CONF_TYPE): vol.In(CONDITION_TYPES),
