@@ -1,6 +1,8 @@
 """Runtime entry data for ESPHome stored in hass.data."""
+from __future__ import annotations
+
 import asyncio
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Set, Tuple
+from typing import TYPE_CHECKING, Any, Callable
 
 from aioesphomeapi import (
     COMPONENT_TYPE_TO_INFO,
@@ -21,10 +23,9 @@ from aioesphomeapi import (
 import attr
 
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_send
 from homeassistant.helpers.storage import Store
-from homeassistant.helpers.typing import HomeAssistantType
 
 if TYPE_CHECKING:
     from . import APIClient
@@ -49,30 +50,31 @@ INFO_TYPE_TO_PLATFORM = {
 class RuntimeEntryData:
     """Store runtime data for esphome config entries."""
 
+    _storage_contents: dict | None = None
+
     entry_id: str = attr.ib()
-    client: "APIClient" = attr.ib()
+    client: APIClient = attr.ib()
     store: Store = attr.ib()
-    reconnect_task: Optional[asyncio.Task] = attr.ib(default=None)
-    state: Dict[str, Dict[str, Any]] = attr.ib(factory=dict)
-    info: Dict[str, Dict[str, Any]] = attr.ib(factory=dict)
+    state: dict[str, dict[str, Any]] = attr.ib(factory=dict)
+    info: dict[str, dict[str, Any]] = attr.ib(factory=dict)
 
     # A second list of EntityInfo objects
     # This is necessary for when an entity is being removed. HA requires
     # some static info to be accessible during removal (unique_id, maybe others)
     # If an entity can't find anything in the info array, it will look for info here.
-    old_info: Dict[str, Dict[str, Any]] = attr.ib(factory=dict)
+    old_info: dict[str, dict[str, Any]] = attr.ib(factory=dict)
 
-    services: Dict[int, "UserService"] = attr.ib(factory=dict)
+    services: dict[int, UserService] = attr.ib(factory=dict)
     available: bool = attr.ib(default=False)
-    device_info: Optional[DeviceInfo] = attr.ib(default=None)
-    cleanup_callbacks: List[Callable[[], None]] = attr.ib(factory=list)
-    disconnect_callbacks: List[Callable[[], None]] = attr.ib(factory=list)
-    loaded_platforms: Set[str] = attr.ib(factory=set)
+    device_info: DeviceInfo | None = attr.ib(default=None)
+    cleanup_callbacks: list[Callable[[], None]] = attr.ib(factory=list)
+    disconnect_callbacks: list[Callable[[], None]] = attr.ib(factory=list)
+    loaded_platforms: set[str] = attr.ib(factory=set)
     platform_load_lock: asyncio.Lock = attr.ib(factory=asyncio.Lock)
 
     @callback
     def async_update_entity(
-        self, hass: HomeAssistantType, component_key: str, key: int
+        self, hass: HomeAssistant, component_key: str, key: int
     ) -> None:
         """Schedule the update of an entity."""
         signal = f"esphome_{self.entry_id}_update_{component_key}_{key}"
@@ -80,14 +82,14 @@ class RuntimeEntryData:
 
     @callback
     def async_remove_entity(
-        self, hass: HomeAssistantType, component_key: str, key: int
+        self, hass: HomeAssistant, component_key: str, key: int
     ) -> None:
         """Schedule the removal of an entity."""
         signal = f"esphome_{self.entry_id}_remove_{component_key}_{key}"
         async_dispatcher_send(hass, signal)
 
     async def _ensure_platforms_loaded(
-        self, hass: HomeAssistantType, entry: ConfigEntry, platforms: Set[str]
+        self, hass: HomeAssistant, entry: ConfigEntry, platforms: set[str]
     ):
         async with self.platform_load_lock:
             needed = platforms - self.loaded_platforms
@@ -101,7 +103,7 @@ class RuntimeEntryData:
             self.loaded_platforms |= needed
 
     async def async_update_static_infos(
-        self, hass: HomeAssistantType, entry: ConfigEntry, infos: List[EntityInfo]
+        self, hass: HomeAssistant, entry: ConfigEntry, infos: list[EntityInfo]
     ) -> None:
         """Distribute an update of static infos to all platforms."""
         # First, load all platforms
@@ -118,22 +120,23 @@ class RuntimeEntryData:
         async_dispatcher_send(hass, signal, infos)
 
     @callback
-    def async_update_state(self, hass: HomeAssistantType, state: EntityState) -> None:
+    def async_update_state(self, hass: HomeAssistant, state: EntityState) -> None:
         """Distribute an update of state information to all platforms."""
         signal = f"esphome_{self.entry_id}_on_state"
         async_dispatcher_send(hass, signal, state)
 
     @callback
-    def async_update_device_state(self, hass: HomeAssistantType) -> None:
+    def async_update_device_state(self, hass: HomeAssistant) -> None:
         """Distribute an update of a core device state like availability."""
         signal = f"esphome_{self.entry_id}_on_device_update"
         async_dispatcher_send(hass, signal)
 
-    async def async_load_from_store(self) -> Tuple[List[EntityInfo], List[UserService]]:
+    async def async_load_from_store(self) -> tuple[list[EntityInfo], list[UserService]]:
         """Load the retained data from store and return de-serialized data."""
         restored = await self.store.async_load()
         if restored is None:
             return [], []
+        self._storage_contents = restored.copy()
 
         self.device_info = _attr_obj_from_dict(
             DeviceInfo, **restored.pop("device_info")
@@ -159,7 +162,14 @@ class RuntimeEntryData:
         for service in self.services.values():
             store_data["services"].append(service.to_dict())
 
-        self.store.async_delay_save(lambda: store_data, SAVE_DELAY)
+        if store_data == self._storage_contents:
+            return
+
+        def _memorized_storage():
+            self._storage_contents = store_data
+            return store_data
+
+        self.store.async_delay_save(_memorized_storage, SAVE_DELAY)
 
 
 def _attr_obj_from_dict(cls, **kwargs):
