@@ -1,9 +1,8 @@
 """The Garmin Connect integration."""
-import asyncio
-from datetime import date, timedelta
+from datetime import date
 import logging
 
-from garminconnect import (
+from garminconnect_ha import (
     Garmin,
     GarminConnectAuthenticationError,
     GarminConnectConnectionError,
@@ -16,29 +15,23 @@ from homeassistant.core import HomeAssistant
 from homeassistant.exceptions import ConfigEntryNotReady
 from homeassistant.util import Throttle
 
-from .const import DOMAIN
+from .const import DEFAULT_UPDATE_INTERVAL, DOMAIN
 
 _LOGGER = logging.getLogger(__name__)
 
 PLATFORMS = ["sensor"]
-MIN_SCAN_INTERVAL = timedelta(minutes=10)
-
-
-async def async_setup(hass: HomeAssistant, config: dict):
-    """Set up the Garmin Connect component."""
-    hass.data[DOMAIN] = {}
-    return True
 
 
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     """Set up Garmin Connect from a config entry."""
-    username = entry.data[CONF_USERNAME]
-    password = entry.data[CONF_PASSWORD]
 
-    garmin_client = Garmin(username, password)
+    username: str = entry.data[CONF_USERNAME]
+    password: str = entry.data[CONF_PASSWORD]
+
+    api = Garmin(username, password)
 
     try:
-        garmin_client.login()
+        await hass.async_add_executor_job(api.login)
     except (
         GarminConnectAuthenticationError,
         GarminConnectTooManyRequestsError,
@@ -49,35 +42,25 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         _LOGGER.error(
             "Connection error occurred during Garmin Connect login request: %s", err
         )
-        raise ConfigEntryNotReady
+        raise ConfigEntryNotReady from err
     except Exception:  # pylint: disable=broad-except
         _LOGGER.exception("Unknown error occurred during Garmin Connect login request")
         return False
 
-    garmin_data = GarminConnectData(hass, garmin_client)
+    garmin_data = GarminConnectData(hass, api)
+    hass.data.setdefault(DOMAIN, {})
     hass.data[DOMAIN][entry.entry_id] = garmin_data
 
-    for component in PLATFORMS:
-        hass.async_create_task(
-            hass.config_entries.async_forward_entry_setup(entry, component)
-        )
+    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
 
     return True
 
 
 async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
     """Unload a config entry."""
-    unload_ok = all(
-        await asyncio.gather(
-            *[
-                hass.config_entries.async_forward_entry_unload(entry, component)
-                for component in PLATFORMS
-            ]
-        )
-    )
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
     if unload_ok:
         hass.data[DOMAIN].pop(entry.entry_id)
-
     return unload_ok
 
 
@@ -86,27 +69,39 @@ class GarminConnectData:
 
     def __init__(self, hass, client):
         """Initialize."""
+        self.hass = hass
         self.client = client
         self.data = None
 
-    @Throttle(MIN_SCAN_INTERVAL)
+    @Throttle(DEFAULT_UPDATE_INTERVAL)
     async def async_update(self):
-        """Update data via library."""
+        """Update data via API wrapper."""
         today = date.today()
 
         try:
-            self.data = self.client.get_stats_and_body(today.isoformat())
+            summary = await self.hass.async_add_executor_job(
+                self.client.get_user_summary, today.isoformat()
+            )
+            body = await self.hass.async_add_executor_job(
+                self.client.get_body_composition, today.isoformat()
+            )
+
+            self.data = {
+                **summary,
+                **body["totalAverage"],
+            }
+            self.data["nextAlarm"] = await self.hass.async_add_executor_job(
+                self.client.get_device_alarms
+            )
         except (
             GarminConnectAuthenticationError,
             GarminConnectTooManyRequestsError,
             GarminConnectConnectionError,
         ) as err:
             _LOGGER.error(
-                "Error occurred during Garmin Connect get activity request: %s", err
+                "Error occurred during Garmin Connect update requests: %s", err
             )
-            return
         except Exception:  # pylint: disable=broad-except
             _LOGGER.exception(
-                "Unknown error occurred during Garmin Connect get activity request"
+                "Unknown error occurred during Garmin Connect update requests"
             )
-            return
