@@ -7,7 +7,9 @@ from homeassistant.components import (
     cover,
     fan,
     group,
+    humidifier,
     input_boolean,
+    input_select,
     light,
     lock,
     media_player,
@@ -28,9 +30,7 @@ CONF_EXPOSE_BY_DEFAULT = "expose_by_default"
 CONF_EXPOSED_DOMAINS = "exposed_domains"
 CONF_PROJECT_ID = "project_id"
 CONF_ALIASES = "aliases"
-CONF_API_KEY = "api_key"
 CONF_ROOM_HINT = "room"
-CONF_ALLOW_UNLOCK = "allow_unlock"
 CONF_SECURE_DEVICES_PIN = "secure_devices_pin"
 CONF_REPORT_STATE = "report_state"
 CONF_SERVICE_ACCOUNT = "service_account"
@@ -43,7 +43,9 @@ DEFAULT_EXPOSED_DOMAINS = [
     "cover",
     "fan",
     "group",
+    "humidifier",
     "input_boolean",
+    "input_select",
     "light",
     "media_player",
     "scene",
@@ -65,7 +67,9 @@ TYPE_SCENE = f"{PREFIX_TYPES}SCENE"
 TYPE_FAN = f"{PREFIX_TYPES}FAN"
 TYPE_THERMOSTAT = f"{PREFIX_TYPES}THERMOSTAT"
 TYPE_LOCK = f"{PREFIX_TYPES}LOCK"
+TYPE_AWNING = f"{PREFIX_TYPES}AWNING"
 TYPE_BLINDS = f"{PREFIX_TYPES}BLINDS"
+TYPE_SHUTTER = f"{PREFIX_TYPES}SHUTTER"
 TYPE_GARAGE = f"{PREFIX_TYPES}GARAGE"
 TYPE_OUTLET = f"{PREFIX_TYPES}OUTLET"
 TYPE_SENSOR = f"{PREFIX_TYPES}SENSOR"
@@ -73,6 +77,10 @@ TYPE_DOOR = f"{PREFIX_TYPES}DOOR"
 TYPE_TV = f"{PREFIX_TYPES}TV"
 TYPE_SPEAKER = f"{PREFIX_TYPES}SPEAKER"
 TYPE_ALARM = f"{PREFIX_TYPES}SECURITYSYSTEM"
+TYPE_SETTOP = f"{PREFIX_TYPES}SETTOP"
+TYPE_HUMIDIFIER = f"{PREFIX_TYPES}HUMIDIFIER"
+TYPE_DEHUMIDIFIER = f"{PREFIX_TYPES}DEHUMIDIFIER"
+TYPE_RECEIVER = f"{PREFIX_TYPES}AUDIO_VIDEO_RECEIVER"
 
 SERVICE_REQUEST_SYNC = "request_sync"
 HOMEGRAPH_URL = "https://homegraph.googleapis.com/"
@@ -90,9 +98,12 @@ ERR_NOT_SUPPORTED = "notSupported"
 ERR_PROTOCOL_ERROR = "protocolError"
 ERR_UNKNOWN_ERROR = "unknownError"
 ERR_FUNCTION_NOT_SUPPORTED = "functionNotSupported"
+ERR_UNSUPPORTED_INPUT = "unsupportedInput"
+ERR_NO_AVAILABLE_CHANNEL = "noAvailableChannel"
 
 ERR_ALREADY_DISARMED = "alreadyDisarmed"
 ERR_ALREADY_ARMED = "alreadyArmed"
+ERR_ALREADY_STOPPED = "alreadyStopped"
 
 ERR_CHALLENGE_NEEDED = "challengeNeeded"
 ERR_CHALLENGE_NOT_SETUP = "challengeFailedNotSetup"
@@ -111,10 +122,12 @@ DOMAIN_TO_GOOGLE_TYPES = {
     cover.DOMAIN: TYPE_BLINDS,
     fan.DOMAIN: TYPE_FAN,
     group.DOMAIN: TYPE_SWITCH,
+    humidifier.DOMAIN: TYPE_HUMIDIFIER,
     input_boolean.DOMAIN: TYPE_SWITCH,
+    input_select.DOMAIN: TYPE_SENSOR,
     light.DOMAIN: TYPE_LIGHT,
     lock.DOMAIN: TYPE_LOCK,
-    media_player.DOMAIN: TYPE_SWITCH,
+    media_player.DOMAIN: TYPE_SETTOP,
     scene.DOMAIN: TYPE_SCENE,
     script.DOMAIN: TYPE_SCENE,
     switch.DOMAIN: TYPE_SWITCH,
@@ -126,6 +139,8 @@ DEVICE_CLASS_TO_GOOGLE_TYPES = {
     (cover.DOMAIN, cover.DEVICE_CLASS_GARAGE): TYPE_GARAGE,
     (cover.DOMAIN, cover.DEVICE_CLASS_GATE): TYPE_GARAGE,
     (cover.DOMAIN, cover.DEVICE_CLASS_DOOR): TYPE_DOOR,
+    (cover.DOMAIN, cover.DEVICE_CLASS_AWNING): TYPE_AWNING,
+    (cover.DOMAIN, cover.DEVICE_CLASS_SHUTTER): TYPE_SHUTTER,
     (switch.DOMAIN, switch.DEVICE_CLASS_SWITCH): TYPE_SWITCH,
     (switch.DOMAIN, switch.DEVICE_CLASS_OUTLET): TYPE_OUTLET,
     (binary_sensor.DOMAIN, binary_sensor.DEVICE_CLASS_DOOR): TYPE_DOOR,
@@ -134,8 +149,12 @@ DEVICE_CLASS_TO_GOOGLE_TYPES = {
     (binary_sensor.DOMAIN, binary_sensor.DEVICE_CLASS_OPENING): TYPE_SENSOR,
     (binary_sensor.DOMAIN, binary_sensor.DEVICE_CLASS_WINDOW): TYPE_SENSOR,
     (media_player.DOMAIN, media_player.DEVICE_CLASS_TV): TYPE_TV,
+    (media_player.DOMAIN, media_player.DEVICE_CLASS_SPEAKER): TYPE_SPEAKER,
+    (media_player.DOMAIN, media_player.DEVICE_CLASS_RECEIVER): TYPE_RECEIVER,
     (sensor.DOMAIN, sensor.DEVICE_CLASS_TEMPERATURE): TYPE_SENSOR,
     (sensor.DOMAIN, sensor.DEVICE_CLASS_HUMIDITY): TYPE_SENSOR,
+    (humidifier.DOMAIN, humidifier.DEVICE_CLASS_HUMIDIFIER): TYPE_HUMIDIFIER,
+    (humidifier.DOMAIN, humidifier.DEVICE_CLASS_DEHUMIDIFIER): TYPE_DEHUMIDIFIER,
 }
 
 CHALLENGE_ACK_NEEDED = "ackNeeded"
