@@ -1,9 +1,9 @@
 """The Hunter Douglas PowerView integration."""
-import asyncio
 from datetime import timedelta
 import logging
 
 from aiopvapi.helpers.aiorequest import AioRequest
+from aiopvapi.helpers.api_base import ApiEntryPoint
 from aiopvapi.helpers.constants import ATTR_ID
 from aiopvapi.helpers.tools import base64_to_unicode
 from aiopvapi.rooms import Rooms
@@ -21,7 +21,9 @@ import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator, UpdateFailed
 
 from .const import (
+    API_PATH_FWVERSION,
     COORDINATOR,
+    DEFAULT_LEGACY_MAINPROCESSOR,
     DEVICE_FIRMWARE,
     DEVICE_INFO,
     DEVICE_MAC_ADDRESS,
@@ -30,24 +32,18 @@ from .const import (
     DEVICE_REVISION,
     DEVICE_SERIAL_NUMBER,
     DOMAIN,
-    FIRMWARE_BUILD,
-    FIRMWARE_IN_USERDATA,
-    FIRMWARE_SUB_REVISION,
+    FIRMWARE,
+    FIRMWARE_MAINPROCESSOR,
+    FIRMWARE_NAME,
+    FIRMWARE_REVISION,
     HUB_EXCEPTIONS,
     HUB_NAME,
-    LEGACY_DEVICE_BUILD,
-    LEGACY_DEVICE_MODEL,
-    LEGACY_DEVICE_REVISION,
-    LEGACY_DEVICE_SUB_REVISION,
     MAC_ADDRESS_IN_USERDATA,
-    MAINPROCESSOR_IN_USERDATA_FIRMWARE,
-    MODEL_IN_MAINPROCESSOR,
     PV_API,
     PV_ROOM_DATA,
     PV_SCENE_DATA,
     PV_SHADE_DATA,
     PV_SHADES,
-    REVISION_IN_MAINPROCESSOR,
     ROOM_DATA,
     SCENE_DATA,
     SERIAL_NUMBER_IN_USERDATA,
@@ -63,12 +59,6 @@ PLATFORMS = ["cover", "scene", "sensor"]
 _LOGGER = logging.getLogger(__name__)
 
 
-async def async_setup(hass: HomeAssistant, hass_config: dict):
-    """Set up the Hunter Douglas PowerView component."""
-    hass.data.setdefault(DOMAIN, {})
-    return True
-
-
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     """Set up Hunter Douglas PowerView from a config entry."""
 
@@ -122,6 +112,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         update_interval=timedelta(seconds=60),
     )
 
+    hass.data.setdefault(DOMAIN, {})
     hass.data[DOMAIN][entry.entry_id] = {
         PV_API: pv_request,
         PV_ROOM_DATA: room_data,
@@ -132,10 +123,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         DEVICE_INFO: device_info,
     }
 
-    for component in PLATFORMS:
-        hass.async_create_task(
-            hass.config_entries.async_forward_entry_setup(entry, component)
-        )
+    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
 
     return True
 
@@ -146,26 +134,25 @@ async def async_get_device_info(pv_request):
     resources = await userdata.get_resources()
     userdata_data = resources[USER_DATA]
 
-    if FIRMWARE_IN_USERDATA in userdata_data:
-        main_processor_info = userdata_data[FIRMWARE_IN_USERDATA][
-            MAINPROCESSOR_IN_USERDATA_FIRMWARE
-        ]
-    else:
+    if FIRMWARE in userdata_data:
+        main_processor_info = userdata_data[FIRMWARE][FIRMWARE_MAINPROCESSOR]
+    elif userdata_data:
         # Legacy devices
-        main_processor_info = {
-            REVISION_IN_MAINPROCESSOR: LEGACY_DEVICE_REVISION,
-            FIRMWARE_SUB_REVISION: LEGACY_DEVICE_SUB_REVISION,
-            FIRMWARE_BUILD: LEGACY_DEVICE_BUILD,
-            MODEL_IN_MAINPROCESSOR: LEGACY_DEVICE_MODEL,
-        }
+        fwversion = ApiEntryPoint(pv_request, API_PATH_FWVERSION)
+        resources = await fwversion.get_resources()
+
+        if FIRMWARE in resources:
+            main_processor_info = resources[FIRMWARE][FIRMWARE_MAINPROCESSOR]
+        else:
+            main_processor_info = DEFAULT_LEGACY_MAINPROCESSOR
 
     return {
         DEVICE_NAME: base64_to_unicode(userdata_data[HUB_NAME]),
         DEVICE_MAC_ADDRESS: userdata_data[MAC_ADDRESS_IN_USERDATA],
         DEVICE_SERIAL_NUMBER: userdata_data[SERIAL_NUMBER_IN_USERDATA],
-        DEVICE_REVISION: main_processor_info[REVISION_IN_MAINPROCESSOR],
+        DEVICE_REVISION: main_processor_info[FIRMWARE_REVISION],
         DEVICE_FIRMWARE: main_processor_info,
-        DEVICE_MODEL: main_processor_info[MODEL_IN_MAINPROCESSOR],
+        DEVICE_MODEL: main_processor_info[FIRMWARE_NAME],
     }
 
 
@@ -177,15 +164,7 @@ def _async_map_data_by_id(data):
 
 async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
     """Unload a config entry."""
-    unload_ok = all(
-        await asyncio.gather(
-            *[
-                hass.config_entries.async_forward_entry_unload(entry, component)
-                for component in PLATFORMS
-            ]
-        )
-    )
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
     if unload_ok:
         hass.data[DOMAIN].pop(entry.entry_id)
-
     return unload_ok
