@@ -1,4 +1,6 @@
 """Support KNX devices."""
+from __future__ import annotations
+
 import asyncio
 import logging
 
@@ -13,27 +15,27 @@ from xknx.io import (
     ConnectionConfig,
     ConnectionType,
 )
-from xknx.telegram import AddressFilter, GroupAddress, Telegram
+from xknx.telegram import AddressFilter, Telegram
+from xknx.telegram.address import parse_device_group_address
 from xknx.telegram.apci import GroupValueRead, GroupValueResponse, GroupValueWrite
 
 from homeassistant.const import (
-    CONF_ADDRESS,
     CONF_HOST,
     CONF_PORT,
     EVENT_HOMEASSISTANT_STOP,
     SERVICE_RELOAD,
 )
+from homeassistant.core import Event, HomeAssistant, ServiceCall
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers import discovery
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity_platform import async_get_platforms
 from homeassistant.helpers.reload import async_integration_yaml_config
 from homeassistant.helpers.service import async_register_admin_service
-from homeassistant.helpers.typing import ServiceCallType
+from homeassistant.helpers.typing import ConfigType
 
-from .const import DOMAIN, SupportedPlatforms
-from .expose import create_knx_exposure
-from .factory import create_knx_device
+from .const import DOMAIN, KNX_ADDRESS, SupportedPlatforms
+from .expose import KNXExposeSensor, KNXExposeTime, create_knx_exposure
 from .schema import (
     BinarySensorSchema,
     ClimateSchema,
@@ -49,12 +51,11 @@ from .schema import (
     WeatherSchema,
     ga_validator,
     ia_validator,
+    sensor_type_validator,
 )
 
 _LOGGER = logging.getLogger(__name__)
 
-CONF_KNX_CONFIG = "config_file"
-
 CONF_KNX_ROUTING = "routing"
 CONF_KNX_TUNNELING = "tunneling"
 CONF_KNX_FIRE_EVENT = "fire_event"
@@ -77,11 +78,13 @@ SERVICE_KNX_READ = "read"
 CONFIG_SCHEMA = vol.Schema(
     {
         DOMAIN: vol.All(
+            # deprecated since 2021.4
+            cv.deprecated("config_file"),
+            # deprecated since 2021.2
             cv.deprecated(CONF_KNX_FIRE_EVENT),
             cv.deprecated("fire_event_filter", replacement_key=CONF_KNX_EVENT_FILTER),
             vol.Schema(
                 {
-                    vol.Optional(CONF_KNX_CONFIG): cv.string,
                     vol.Exclusive(
                         CONF_KNX_ROUTING, "connection_type"
                     ): ConnectionSchema.ROUTING_SCHEMA,
@@ -148,15 +151,21 @@ CONFIG_SCHEMA = vol.Schema(
 SERVICE_KNX_SEND_SCHEMA = vol.Any(
     vol.Schema(
         {
-            vol.Required(CONF_ADDRESS): ga_validator,
+            vol.Required(KNX_ADDRESS): vol.All(
+                cv.ensure_list,
+                [ga_validator],
+            ),
             vol.Required(SERVICE_KNX_ATTR_PAYLOAD): cv.match_all,
-            vol.Required(SERVICE_KNX_ATTR_TYPE): vol.Any(int, float, str),
+            vol.Required(SERVICE_KNX_ATTR_TYPE): sensor_type_validator,
         }
     ),
     vol.Schema(
         # without type given payload is treated as raw bytes
         {
-            vol.Required(CONF_ADDRESS): ga_validator,
+            vol.Required(KNX_ADDRESS): vol.All(
+                cv.ensure_list,
+                [ga_validator],
+            ),
             vol.Required(SERVICE_KNX_ATTR_PAYLOAD): vol.Any(
                 cv.positive_int, [cv.positive_int]
             ),
@@ -166,7 +175,7 @@ SERVICE_KNX_SEND_SCHEMA = vol.Any(
 
 SERVICE_KNX_READ_SCHEMA = vol.Schema(
     {
-        vol.Required(CONF_ADDRESS): vol.All(
+        vol.Required(KNX_ADDRESS): vol.All(
             cv.ensure_list,
             [ga_validator],
         )
@@ -175,13 +184,16 @@ SERVICE_KNX_READ_SCHEMA = vol.Schema(
 
 SERVICE_KNX_EVENT_REGISTER_SCHEMA = vol.Schema(
     {
-        vol.Required(CONF_ADDRESS): ga_validator,
+        vol.Required(KNX_ADDRESS): vol.All(
+            cv.ensure_list,
+            [ga_validator],
+        ),
         vol.Optional(SERVICE_KNX_ATTR_REMOVE, default=False): cv.boolean,
     }
 )
 
 SERVICE_KNX_EXPOSURE_REGISTER_SCHEMA = vol.Any(
-    ExposeSchema.SCHEMA.extend(
+    ExposeSchema.EXPOSE_SENSOR_SCHEMA.extend(
         {
             vol.Optional(SERVICE_KNX_ATTR_REMOVE, default=False): cv.boolean,
         }
@@ -189,7 +201,7 @@ SERVICE_KNX_EXPOSURE_REGISTER_SCHEMA = vol.Any(
     vol.Schema(
         # for removing only `address` is required
         {
-            vol.Required(CONF_ADDRESS): ga_validator,
+            vol.Required(KNX_ADDRESS): ga_validator,
             vol.Required(SERVICE_KNX_ATTR_REMOVE): vol.All(cv.boolean, True),
         },
         extra=vol.ALLOW_EXTRA,
@@ -197,8 +209,8 @@ SERVICE_KNX_EXPOSURE_REGISTER_SCHEMA = vol.Any(
 )
 
 
-async def async_setup(hass, config):
-    """Set up the KNX component."""
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
+    """Set up the KNX integration."""
     try:
         knx_module = KNXModule(hass, config)
         hass.data[DOMAIN] = knx_module
@@ -216,20 +228,18 @@ async def async_setup(hass, config):
             )
 
     for platform in SupportedPlatforms:
-        if platform.value in config[DOMAIN]:
-            for device_config in config[DOMAIN][platform.value]:
-                create_knx_device(platform, knx_module.xknx, device_config)
-
-    # We need to wait until all entities are loaded into the device list since they could also be created from other platforms
-    for platform in SupportedPlatforms:
+        if platform.value not in config[DOMAIN]:
+            continue
         hass.async_create_task(
-            discovery.async_load_platform(hass, platform.value, DOMAIN, {}, config)
-        )
-
-    if not knx_module.xknx.devices:
-        _LOGGER.warning(
-            "No KNX devices are configured. Please read "
-            "https://www.home-assistant.io/blog/2020/09/17/release-115/#breaking-changes"
+            discovery.async_load_platform(
+                hass,
+                platform.value,
+                DOMAIN,
+                {
+                    "platform_config": config[DOMAIN][platform.value],
+                },
+                config,
+            )
         )
 
     hass.services.async_register(
@@ -262,7 +272,7 @@ async def async_setup(hass, config):
         schema=SERVICE_KNX_EXPOSURE_REGISTER_SCHEMA,
     )
 
-    async def reload_service_handler(service_call: ServiceCallType) -> None:
+    async def reload_service_handler(service_call: ServiceCall) -> None:
         """Remove all KNX components and load new ones from config."""
 
         # First check for config file. If for some reason it is no longer there
@@ -290,21 +300,20 @@ async def async_setup(hass, config):
 class KNXModule:
     """Representation of KNX Object."""
 
-    def __init__(self, hass, config):
-        """Initialize of KNX module."""
+    def __init__(self, hass: HomeAssistant, config: ConfigType) -> None:
+        """Initialize KNX module."""
         self.hass = hass
         self.config = config
         self.connected = False
-        self.exposures = []
-        self.service_exposures = {}
+        self.exposures: list[KNXExposeSensor | KNXExposeTime] = []
+        self.service_exposures: dict[str, KNXExposeSensor | KNXExposeTime] = {}
 
         self.init_xknx()
         self._knx_event_callback: TelegramQueue.Callback = self.register_callback()
 
-    def init_xknx(self):
-        """Initialize of KNX object."""
+    def init_xknx(self) -> None:
+        """Initialize XKNX object."""
         self.xknx = XKNX(
-            config=self.config_file(),
             own_address=self.config[DOMAIN][CONF_KNX_INDIVIDUAL_ADDRESS],
             rate_limit=self.config[DOMAIN][CONF_KNX_RATE_LIMIT],
             multicast_group=self.config[DOMAIN][CONF_KNX_MCAST_GRP],
@@ -313,26 +322,17 @@ class KNXModule:
             state_updater=self.config[DOMAIN][CONF_KNX_STATE_UPDATER],
         )
 
-    async def start(self):
-        """Start KNX object. Connect to tunneling or Routing device."""
+    async def start(self) -> None:
+        """Start XKNX object. Connect to tunneling or Routing device."""
         await self.xknx.start()
         self.hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, self.stop)
         self.connected = True
 
-    async def stop(self, event):
-        """Stop KNX object. Disconnect from tunneling or Routing device."""
+    async def stop(self, event: Event) -> None:
+        """Stop XKNX object. Disconnect from tunneling or Routing device."""
         await self.xknx.stop()
 
-    def config_file(self):
-        """Resolve and return the full path of xknx.yaml if configured."""
-        config_file = self.config[DOMAIN].get(CONF_KNX_CONFIG)
-        if not config_file:
-            return None
-        if not config_file.startswith("/"):
-            return self.hass.config.path(config_file)
-        return config_file
-
-    def connection_config(self):
+    def connection_config(self) -> ConnectionConfig:
         """Return the connection_config."""
         if CONF_KNX_TUNNELING in self.config[DOMAIN]:
             return self.connection_config_tunneling()
@@ -341,7 +341,7 @@ class KNXModule:
         # config from xknx.yaml always has priority later on
         return ConnectionConfig(auto_reconnect=True)
 
-    def connection_config_routing(self):
+    def connection_config_routing(self) -> ConnectionConfig:
         """Return the connection_config if routing is configured."""
         local_ip = None
         # all configuration values are optional
@@ -353,27 +353,33 @@ class KNXModule:
             connection_type=ConnectionType.ROUTING, local_ip=local_ip
         )
 
-    def connection_config_tunneling(self):
+    def connection_config_tunneling(self) -> ConnectionConfig:
         """Return the connection_config if tunneling is configured."""
         gateway_ip = self.config[DOMAIN][CONF_KNX_TUNNELING][CONF_HOST]
         gateway_port = self.config[DOMAIN][CONF_KNX_TUNNELING][CONF_PORT]
         local_ip = self.config[DOMAIN][CONF_KNX_TUNNELING].get(
             ConnectionSchema.CONF_KNX_LOCAL_IP
         )
+        route_back = self.config[DOMAIN][CONF_KNX_TUNNELING][
+            ConnectionSchema.CONF_KNX_ROUTE_BACK
+        ]
         return ConnectionConfig(
             connection_type=ConnectionType.TUNNELING,
             gateway_ip=gateway_ip,
             gateway_port=gateway_port,
             local_ip=local_ip,
+            route_back=route_back,
             auto_reconnect=True,
         )
 
-    async def telegram_received_cb(self, telegram):
+    async def telegram_received_cb(self, telegram: Telegram) -> None:
         """Call invoked after a KNX telegram was received."""
         data = None
-
         # Not all telegrams have serializable data.
-        if isinstance(telegram.payload, (GroupValueWrite, GroupValueResponse)):
+        if (
+            isinstance(telegram.payload, (GroupValueWrite, GroupValueResponse))
+            and telegram.payload.value is not None
+        ):
             data = telegram.payload.value.value
 
         self.hass.bus.async_fire(
@@ -399,27 +405,32 @@ class KNXModule:
             match_for_outgoing=True,
         )
 
-    async def service_event_register_modify(self, call):
+    async def service_event_register_modify(self, call: ServiceCall) -> None:
         """Service for adding or removing a GroupAddress to the knx_event filter."""
-        group_address = GroupAddress(call.data[CONF_ADDRESS])
-        if call.data.get(SERVICE_KNX_ATTR_REMOVE):
-            try:
-                self._knx_event_callback.group_addresses.remove(group_address)
-            except ValueError:
-                _LOGGER.warning(
-                    "Service event_register could not remove event for '%s'",
-                    group_address,
-                )
-        elif group_address not in self._knx_event_callback.group_addresses:
-            self._knx_event_callback.group_addresses.append(group_address)
-            _LOGGER.debug(
-                "Service event_register registered event for '%s'",
-                group_address,
-            )
+        attr_address = call.data[KNX_ADDRESS]
+        group_addresses = map(parse_device_group_address, attr_address)
 
-    async def service_exposure_register_modify(self, call):
+        if call.data.get(SERVICE_KNX_ATTR_REMOVE):
+            for group_address in group_addresses:
+                try:
+                    self._knx_event_callback.group_addresses.remove(group_address)
+                except ValueError:
+                    _LOGGER.warning(
+                        "Service event_register could not remove event for '%s'",
+                        str(group_address),
+                    )
+        else:
+            for group_address in group_addresses:
+                if group_address not in self._knx_event_callback.group_addresses:
+                    self._knx_event_callback.group_addresses.append(group_address)
+                    _LOGGER.debug(
+                        "Service event_register registered event for '%s'",
+                        str(group_address),
+                    )
+
+    async def service_exposure_register_modify(self, call: ServiceCall) -> None:
         """Service for adding or removing an exposure to KNX bus."""
-        group_address = call.data.get(CONF_ADDRESS)
+        group_address = call.data[KNX_ADDRESS]
 
         if call.data.get(SERVICE_KNX_ATTR_REMOVE):
             try:
@@ -434,13 +445,14 @@ class KNXModule:
 
         if group_address in self.service_exposures:
             replaced_exposure = self.service_exposures.pop(group_address)
+            assert replaced_exposure.device is not None
             _LOGGER.warning(
                 "Service exposure_register replacing already registered exposure for '%s' - %s",
                 group_address,
                 replaced_exposure.device.name,
             )
             replaced_exposure.shutdown()
-        exposure = create_knx_exposure(self.hass, self.xknx, call.data)
+        exposure = create_knx_exposure(self.hass, self.xknx, call.data)  # type: ignore[arg-type]
         self.service_exposures[group_address] = exposure
         _LOGGER.debug(
             "Service exposure_register registered exposure for '%s' - %s",
@@ -448,34 +460,35 @@ class KNXModule:
             exposure.device.name,
         )
 
-    async def service_send_to_knx_bus(self, call):
+    async def service_send_to_knx_bus(self, call: ServiceCall) -> None:
         """Service for sending an arbitrary KNX message to the KNX bus."""
-        attr_payload = call.data.get(SERVICE_KNX_ATTR_PAYLOAD)
-        attr_address = call.data.get(CONF_ADDRESS)
+        attr_address = call.data[KNX_ADDRESS]
+        attr_payload = call.data[SERVICE_KNX_ATTR_PAYLOAD]
         attr_type = call.data.get(SERVICE_KNX_ATTR_TYPE)
 
-        def calculate_payload(attr_payload):
-            """Calculate payload depending on type of attribute."""
-            if attr_type is not None:
-                transcoder = DPTBase.parse_transcoder(attr_type)
-                if transcoder is None:
-                    raise ValueError(f"Invalid type for knx.send service: {attr_type}")
-                return DPTArray(transcoder.to_knx(attr_payload))
-            if isinstance(attr_payload, int):
-                return DPTBinary(attr_payload)
-            return DPTArray(attr_payload)
+        payload: DPTBinary | DPTArray
+        if attr_type is not None:
+            transcoder = DPTBase.parse_transcoder(attr_type)
+            if transcoder is None:
+                raise ValueError(f"Invalid type for knx.send service: {attr_type}")
+            payload = DPTArray(transcoder.to_knx(attr_payload))
+        elif isinstance(attr_payload, int):
+            payload = DPTBinary(attr_payload)
+        else:
+            payload = DPTArray(attr_payload)
 
-        telegram = Telegram(
-            destination_address=GroupAddress(attr_address),
-            payload=GroupValueWrite(calculate_payload(attr_payload)),
-        )
-        await self.xknx.telegrams.put(telegram)
-
-    async def service_read_to_knx_bus(self, call):
-        """Service for sending a GroupValueRead telegram to the KNX bus."""
-        for address in call.data.get(CONF_ADDRESS):
+        for address in attr_address:
             telegram = Telegram(
-                destination_address=GroupAddress(address),
+                destination_address=parse_device_group_address(address),
+                payload=GroupValueWrite(payload),
+            )
+            await self.xknx.telegrams.put(telegram)
+
+    async def service_read_to_knx_bus(self, call: ServiceCall) -> None:
+        """Service for sending a GroupValueRead telegram to the KNX bus."""
+        for address in call.data[KNX_ADDRESS]:
+            telegram = Telegram(
+                destination_address=parse_device_group_address(address),
                 payload=GroupValueRead(),
             )
             await self.xknx.telegrams.put(telegram)
