@@ -1,7 +1,6 @@
 """Support for Generic Modbus Thermostats."""
 from __future__ import annotations
 
-from datetime import timedelta
 import logging
 import struct
 from typing import Any
@@ -12,28 +11,29 @@ from homeassistant.components.climate.const import (
     SUPPORT_TARGET_TEMPERATURE,
 )
 from homeassistant.const import (
+    CONF_ADDRESS,
     CONF_NAME,
     CONF_OFFSET,
-    CONF_SCAN_INTERVAL,
-    CONF_SLAVE,
     CONF_STRUCTURE,
     CONF_TEMPERATURE_UNIT,
     TEMP_CELSIUS,
     TEMP_FAHRENHEIT,
 )
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.event import async_track_time_interval
+from homeassistant.helpers.restore_state import RestoreEntity
 from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
+from .base_platform import BasePlatform
 from .const import (
     ATTR_TEMPERATURE,
     CALL_TYPE_REGISTER_HOLDING,
-    CALL_TYPE_REGISTER_INPUT,
+    CALL_TYPE_WRITE_REGISTERS,
     CONF_CLIMATES,
     CONF_CURRENT_TEMP,
     CONF_CURRENT_TEMP_REGISTER_TYPE,
     CONF_DATA_COUNT,
     CONF_DATA_TYPE,
+    CONF_INPUT_TYPE,
     CONF_MAX_TEMP,
     CONF_MIN_TEMP,
     CONF_PRECISION,
@@ -46,6 +46,7 @@ from .const import (
 )
 from .modbus import ModbusHub
 
+PARALLEL_UPDATES = 1
 _LOGGER = logging.getLogger(__name__)
 
 
@@ -98,18 +99,18 @@ async def async_setup_platform(
     async_add_entities(entities)
 
 
-class ModbusThermostat(ClimateEntity):
+class ModbusThermostat(BasePlatform, RestoreEntity, ClimateEntity):
     """Representation of a Modbus Thermostat."""
 
     def __init__(
         self,
         hub: ModbusHub,
         config: dict[str, Any],
-    ):
+    ) -> None:
         """Initialize the modbus thermostat."""
-        self._hub: ModbusHub = hub
-        self._name = config[CONF_NAME]
-        self._slave = config.get(CONF_SLAVE)
+        config[CONF_ADDRESS] = "0"
+        config[CONF_INPUT_TYPE] = ""
+        super().__init__(hub, config)
         self._target_temperature_register = config[CONF_TARGET_TEMP]
         self._current_temperature_register = config[CONF_CURRENT_TEMP]
         self._current_temperature_register_type = config[
@@ -122,28 +123,18 @@ class ModbusThermostat(ClimateEntity):
         self._count = config[CONF_DATA_COUNT]
         self._precision = config[CONF_PRECISION]
         self._scale = config[CONF_SCALE]
-        self._scan_interval = timedelta(seconds=config[CONF_SCAN_INTERVAL])
         self._offset = config[CONF_OFFSET]
         self._unit = config[CONF_TEMPERATURE_UNIT]
         self._max_temp = config[CONF_MAX_TEMP]
         self._min_temp = config[CONF_MIN_TEMP]
         self._temp_step = config[CONF_STEP]
-        self._available = True
 
     async def async_added_to_hass(self):
         """Handle entity which will be added."""
-        async_track_time_interval(
-            self.hass, lambda arg: self._update(), self._scan_interval
-        )
-
-    @property
-    def should_poll(self):
-        """Return True if entity has to be polled for state.
-
-        False if entity pushes its state to HA.
-        """
-        # Handle polling directly in this entity
-        return False
+        await self.async_base_added_to_hass()
+        state = await self.async_get_last_state()
+        if state and state.attributes.get(ATTR_TEMPERATURE):
+            self._target_temperature = float(state.attributes[ATTR_TEMPERATURE])
 
     @property
     def supported_features(self):
@@ -160,16 +151,11 @@ class ModbusThermostat(ClimateEntity):
         """Return the possible HVAC modes."""
         return [HVAC_MODE_AUTO]
 
-    def set_hvac_mode(self, hvac_mode: str) -> None:
+    async def async_set_hvac_mode(self, hvac_mode: str) -> None:
         """Set new target hvac mode."""
         # Home Assistant expects this method.
         # We'll keep it here to avoid getting exceptions.
 
-    @property
-    def name(self):
-        """Return the name of the climate device."""
-        return self._name
-
     @property
     def current_temperature(self):
         """Return the current temperature."""
@@ -200,7 +186,7 @@ class ModbusThermostat(ClimateEntity):
         """Return the supported step of target temperature."""
         return self._temp_step
 
-    def set_temperature(self, **kwargs):
+    async def async_set_temperature(self, **kwargs):
         """Set new target temperature."""
         if ATTR_TEMPERATURE not in kwargs:
             return
@@ -209,40 +195,36 @@ class ModbusThermostat(ClimateEntity):
         )
         byte_string = struct.pack(self._structure, target_temperature)
         register_value = struct.unpack(">h", byte_string[0:2])[0]
-        self._available = self._hub.write_registers(
+        result = await self._hub.async_pymodbus_call(
             self._slave,
             self._target_temperature_register,
             register_value,
+            CALL_TYPE_WRITE_REGISTERS,
         )
-        self._update()
+        self._available = result is not None
+        await self.async_update()
 
-    @property
-    def available(self) -> bool:
-        """Return True if entity is available."""
-        return self._available
-
-    def _update(self):
+    async def async_update(self, now=None):
         """Update Target & Current Temperature."""
-        self._target_temperature = self._read_register(
+        # remark "now" is a dummy parameter to avoid problems with
+        # async_track_time_interval
+        self._target_temperature = await self._async_read_register(
             CALL_TYPE_REGISTER_HOLDING, self._target_temperature_register
         )
-        self._current_temperature = self._read_register(
+        self._current_temperature = await self._async_read_register(
             self._current_temperature_register_type, self._current_temperature_register
         )
 
-        self.schedule_update_ha_state()
+        self.async_write_ha_state()
 
-    def _read_register(self, register_type, register) -> float | None:
+    async def _async_read_register(self, register_type, register) -> float | None:
         """Read register using the Modbus hub slave."""
-        if register_type == CALL_TYPE_REGISTER_INPUT:
-            result = self._hub.read_input_registers(self._slave, register, self._count)
-        else:
-            result = self._hub.read_holding_registers(
-                self._slave, register, self._count
-            )
+        result = await self._hub.async_pymodbus_call(
+            self._slave, register, self._count, register_type
+        )
         if result is None:
             self._available = False
-            return
+            return -1
 
         byte_string = b"".join(
             [x.to_bytes(2, byteorder="big") for x in result.registers]
@@ -255,11 +237,11 @@ class ModbusThermostat(ClimateEntity):
             )
             return -1
 
-        val = val[0]
+        val2 = val[0]
         register_value = format(
-            (self._scale * val) + self._offset, f".{self._precision}f"
+            (self._scale * val2) + self._offset, f".{self._precision}f"
         )
-        register_value = float(register_value)
+        register_value2 = float(register_value)
         self._available = True
 
-        return register_value
+        return register_value2
