@@ -1,15 +1,19 @@
 """The nut component."""
 from homeassistant.components.sensor import (
     DEVICE_CLASS_BATTERY,
+    DEVICE_CLASS_HUMIDITY,
     DEVICE_CLASS_POWER,
     DEVICE_CLASS_TEMPERATURE,
+    DEVICE_CLASS_VOLTAGE,
 )
 from homeassistant.const import (
+    ELECTRICAL_CURRENT_AMPERE,
+    ELECTRICAL_VOLT_AMPERE,
     FREQUENCY_HERTZ,
+    PERCENTAGE,
     POWER_WATT,
     TEMP_CELSIUS,
     TIME_SECONDS,
-    UNIT_PERCENTAGE,
     VOLT,
 )
 
@@ -43,36 +47,51 @@ SENSOR_TYPES = {
     "ups.temperature": [
         "UPS Temperature",
         TEMP_CELSIUS,
-        "mdi:thermometer",
+        None,
         DEVICE_CLASS_TEMPERATURE,
     ],
-    "ups.load": ["Load", UNIT_PERCENTAGE, "mdi:gauge", None],
-    "ups.load.high": ["Overload Setting", UNIT_PERCENTAGE, "mdi:gauge", None],
+    "ups.load": ["Load", PERCENTAGE, "mdi:gauge", None],
+    "ups.load.high": ["Overload Setting", PERCENTAGE, "mdi:gauge", None],
     "ups.id": ["System identifier", "", "mdi:information-outline", None],
-    "ups.delay.start": ["Load Restart Delay", TIME_SECONDS, "mdi:timer", None],
-    "ups.delay.reboot": ["UPS Reboot Delay", TIME_SECONDS, "mdi:timer", None],
-    "ups.delay.shutdown": ["UPS Shutdown Delay", TIME_SECONDS, "mdi:timer", None],
-    "ups.timer.start": ["Load Start Timer", TIME_SECONDS, "mdi:timer", None],
-    "ups.timer.reboot": ["Load Reboot Timer", TIME_SECONDS, "mdi:timer", None],
-    "ups.timer.shutdown": ["Load Shutdown Timer", TIME_SECONDS, "mdi:timer", None],
-    "ups.test.interval": ["Self-Test Interval", TIME_SECONDS, "mdi:timer", None],
+    "ups.delay.start": ["Load Restart Delay", TIME_SECONDS, "mdi:timer-outline", None],
+    "ups.delay.reboot": ["UPS Reboot Delay", TIME_SECONDS, "mdi:timer-outline", None],
+    "ups.delay.shutdown": [
+        "UPS Shutdown Delay",
+        TIME_SECONDS,
+        "mdi:timer-outline",
+        None,
+    ],
+    "ups.timer.start": ["Load Start Timer", TIME_SECONDS, "mdi:timer-outline", None],
+    "ups.timer.reboot": ["Load Reboot Timer", TIME_SECONDS, "mdi:timer-outline", None],
+    "ups.timer.shutdown": [
+        "Load Shutdown Timer",
+        TIME_SECONDS,
+        "mdi:timer-outline",
+        None,
+    ],
+    "ups.test.interval": [
+        "Self-Test Interval",
+        TIME_SECONDS,
+        "mdi:timer-outline",
+        None,
+    ],
     "ups.test.result": ["Self-Test Result", "", "mdi:information-outline", None],
     "ups.test.date": ["Self-Test Date", "", "mdi:calendar", None],
     "ups.display.language": ["Language", "", "mdi:information-outline", None],
     "ups.contacts": ["External Contacts", "", "mdi:information-outline", None],
-    "ups.efficiency": ["Efficiency", UNIT_PERCENTAGE, "mdi:gauge", None],
-    "ups.power": ["Current Apparent Power", "VA", "mdi:flash", None],
-    "ups.power.nominal": ["Nominal Power", "VA", "mdi:flash", None],
+    "ups.efficiency": ["Efficiency", PERCENTAGE, "mdi:gauge", None],
+    "ups.power": ["Current Apparent Power", ELECTRICAL_VOLT_AMPERE, "mdi:flash", None],
+    "ups.power.nominal": ["Nominal Power", ELECTRICAL_VOLT_AMPERE, "mdi:flash", None],
     "ups.realpower": [
         "Current Real Power",
         POWER_WATT,
-        "mdi:flash",
+        None,
         DEVICE_CLASS_POWER,
     ],
     "ups.realpower.nominal": [
         "Nominal Real Power",
         POWER_WATT,
-        "mdi:flash",
+        None,
         DEVICE_CLASS_POWER,
     ],
     "ups.beeper.status": ["Beeper Status", "", "mdi:information-outline", None],
@@ -84,43 +103,63 @@ SENSOR_TYPES = {
     "ups.shutdown": ["Shutdown Ability", "", "mdi:information-outline", None],
     "battery.charge": [
         "Battery Charge",
-        UNIT_PERCENTAGE,
-        "mdi:gauge",
+        PERCENTAGE,
+        None,
         DEVICE_CLASS_BATTERY,
     ],
-    "battery.charge.low": ["Low Battery Setpoint", UNIT_PERCENTAGE, "mdi:gauge", None],
+    "battery.charge.low": ["Low Battery Setpoint", PERCENTAGE, "mdi:gauge", None],
     "battery.charge.restart": [
         "Minimum Battery to Start",
-        UNIT_PERCENTAGE,
+        PERCENTAGE,
         "mdi:gauge",
         None,
     ],
     "battery.charge.warning": [
         "Warning Battery Setpoint",
-        UNIT_PERCENTAGE,
+        PERCENTAGE,
         "mdi:gauge",
         None,
     ],
     "battery.charger.status": ["Charging Status", "", "mdi:information-outline", None],
-    "battery.voltage": ["Battery Voltage", VOLT, "mdi:flash", None],
-    "battery.voltage.nominal": ["Nominal Battery Voltage", VOLT, "mdi:flash", None],
-    "battery.voltage.low": ["Low Battery Voltage", VOLT, "mdi:flash", None],
-    "battery.voltage.high": ["High Battery Voltage", VOLT, "mdi:flash", None],
+    "battery.voltage": ["Battery Voltage", VOLT, None, DEVICE_CLASS_VOLTAGE],
+    "battery.voltage.nominal": [
+        "Nominal Battery Voltage",
+        VOLT,
+        None,
+        DEVICE_CLASS_VOLTAGE,
+    ],
+    "battery.voltage.low": ["Low Battery Voltage", VOLT, None, DEVICE_CLASS_VOLTAGE],
+    "battery.voltage.high": ["High Battery Voltage", VOLT, None, DEVICE_CLASS_VOLTAGE],
     "battery.capacity": ["Battery Capacity", "Ah", "mdi:flash", None],
-    "battery.current": ["Battery Current", "A", "mdi:flash", None],
-    "battery.current.total": ["Total Battery Current", "A", "mdi:flash", None],
+    "battery.current": [
+        "Battery Current",
+        ELECTRICAL_CURRENT_AMPERE,
+        "mdi:flash",
+        None,
+    ],
+    "battery.current.total": [
+        "Total Battery Current",
+        ELECTRICAL_CURRENT_AMPERE,
+        "mdi:flash",
+        None,
+    ],
     "battery.temperature": [
         "Battery Temperature",
         TEMP_CELSIUS,
-        "mdi:thermometer",
+        None,
         DEVICE_CLASS_TEMPERATURE,
     ],
-    "battery.runtime": ["Battery Runtime", TIME_SECONDS, "mdi:timer", None],
-    "battery.runtime.low": ["Low Battery Runtime", TIME_SECONDS, "mdi:timer", None],
+    "battery.runtime": ["Battery Runtime", TIME_SECONDS, "mdi:timer-outline", None],
+    "battery.runtime.low": [
+        "Low Battery Runtime",
+        TIME_SECONDS,
+        "mdi:timer-outline",
+        None,
+    ],
     "battery.runtime.restart": [
         "Minimum Battery Runtime to Start",
         TIME_SECONDS,
-        "mdi:timer",
+        "mdi:timer-outline",
         None,
     ],
     "battery.alarm.threshold": [
@@ -145,16 +184,21 @@ SENSOR_TYPES = {
         "mdi:information-outline",
         None,
     ],
-    "input.transfer.low": ["Low Voltage Transfer", VOLT, "mdi:flash", None],
-    "input.transfer.high": ["High Voltage Transfer", VOLT, "mdi:flash", None],
+    "input.transfer.low": ["Low Voltage Transfer", VOLT, None, DEVICE_CLASS_VOLTAGE],
+    "input.transfer.high": ["High Voltage Transfer", VOLT, None, DEVICE_CLASS_VOLTAGE],
     "input.transfer.reason": [
         "Voltage Transfer Reason",
         "",
         "mdi:information-outline",
         None,
     ],
-    "input.voltage": ["Input Voltage", VOLT, "mdi:flash", None],
-    "input.voltage.nominal": ["Nominal Input Voltage", VOLT, "mdi:flash", None],
+    "input.voltage": ["Input Voltage", VOLT, None, DEVICE_CLASS_VOLTAGE],
+    "input.voltage.nominal": [
+        "Nominal Input Voltage",
+        VOLT,
+        None,
+        DEVICE_CLASS_VOLTAGE,
+    ],
     "input.frequency": ["Input Line Frequency", FREQUENCY_HERTZ, "mdi:flash", None],
     "input.frequency.nominal": [
         "Nominal Input Line Frequency",
@@ -168,10 +212,20 @@ SENSOR_TYPES = {
         "mdi:information-outline",
         None,
     ],
-    "output.current": ["Output Current", "A", "mdi:flash", None],
-    "output.current.nominal": ["Nominal Output Current", "A", "mdi:flash", None],
-    "output.voltage": ["Output Voltage", VOLT, "mdi:flash", None],
-    "output.voltage.nominal": ["Nominal Output Voltage", VOLT, "mdi:flash", None],
+    "output.current": ["Output Current", ELECTRICAL_CURRENT_AMPERE, "mdi:flash", None],
+    "output.current.nominal": [
+        "Nominal Output Current",
+        ELECTRICAL_CURRENT_AMPERE,
+        "mdi:flash",
+        None,
+    ],
+    "output.voltage": ["Output Voltage", VOLT, None, DEVICE_CLASS_VOLTAGE],
+    "output.voltage.nominal": [
+        "Nominal Output Voltage",
+        VOLT,
+        None,
+        DEVICE_CLASS_VOLTAGE,
+    ],
     "output.frequency": ["Output Frequency", FREQUENCY_HERTZ, "mdi:flash", None],
     "output.frequency.nominal": [
         "Nominal Output Frequency",
@@ -179,6 +233,18 @@ SENSOR_TYPES = {
         "mdi:flash",
         None,
     ],
+    "ambient.humidity": [
+        "Ambient Humidity",
+        PERCENTAGE,
+        None,
+        DEVICE_CLASS_HUMIDITY,
+    ],
+    "ambient.temperature": [
+        "Ambient Temperature",
+        TEMP_CELSIUS,
+        None,
+        DEVICE_CLASS_TEMPERATURE,
+    ],
 }
 
 STATE_TYPES = {
