@@ -1,12 +1,12 @@
 """Support for OpenUV sensors."""
-import logging
-
-from homeassistant.const import TIME_MINUTES
+from homeassistant.components.sensor import SensorEntity
+from homeassistant.const import TIME_MINUTES, UV_INDEX
 from homeassistant.core import callback
 from homeassistant.util.dt import as_local, parse_datetime
 
-from . import (
-    DATA_OPENUV_CLIENT,
+from . import OpenUvEntity
+from .const import (
+    DATA_CLIENT,
     DATA_UV,
     DOMAIN,
     TYPE_CURRENT_OZONE_LEVEL,
@@ -19,11 +19,8 @@ from . import (
     TYPE_SAFE_EXPOSURE_TIME_4,
     TYPE_SAFE_EXPOSURE_TIME_5,
     TYPE_SAFE_EXPOSURE_TIME_6,
-    OpenUvEntity,
 )
 
-_LOGGER = logging.getLogger(__name__)
-
 ATTR_MAX_UV_TIME = "time"
 
 EXPOSURE_TYPE_MAP = {
@@ -43,45 +40,45 @@ UV_LEVEL_LOW = "Low"
 
 SENSORS = {
     TYPE_CURRENT_OZONE_LEVEL: ("Current Ozone Level", "mdi:vector-triangle", "du"),
-    TYPE_CURRENT_UV_INDEX: ("Current UV Index", "mdi:weather-sunny", "index"),
+    TYPE_CURRENT_UV_INDEX: ("Current UV Index", "mdi:weather-sunny", UV_INDEX),
     TYPE_CURRENT_UV_LEVEL: ("Current UV Level", "mdi:weather-sunny", None),
-    TYPE_MAX_UV_INDEX: ("Max UV Index", "mdi:weather-sunny", "index"),
+    TYPE_MAX_UV_INDEX: ("Max UV Index", "mdi:weather-sunny", UV_INDEX),
     TYPE_SAFE_EXPOSURE_TIME_1: (
         "Skin Type 1 Safe Exposure Time",
-        "mdi:timer",
+        "mdi:timer-outline",
         TIME_MINUTES,
     ),
     TYPE_SAFE_EXPOSURE_TIME_2: (
         "Skin Type 2 Safe Exposure Time",
-        "mdi:timer",
+        "mdi:timer-outline",
         TIME_MINUTES,
     ),
     TYPE_SAFE_EXPOSURE_TIME_3: (
         "Skin Type 3 Safe Exposure Time",
-        "mdi:timer",
+        "mdi:timer-outline",
         TIME_MINUTES,
     ),
     TYPE_SAFE_EXPOSURE_TIME_4: (
         "Skin Type 4 Safe Exposure Time",
-        "mdi:timer",
+        "mdi:timer-outline",
         TIME_MINUTES,
     ),
     TYPE_SAFE_EXPOSURE_TIME_5: (
         "Skin Type 5 Safe Exposure Time",
-        "mdi:timer",
+        "mdi:timer-outline",
         TIME_MINUTES,
     ),
     TYPE_SAFE_EXPOSURE_TIME_6: (
         "Skin Type 6 Safe Exposure Time",
-        "mdi:timer",
+        "mdi:timer-outline",
         TIME_MINUTES,
     ),
 }
 
 
 async def async_setup_entry(hass, entry, async_add_entities):
-    """Set up a Nest sensor based on a config entry."""
-    openuv = hass.data[DOMAIN][DATA_OPENUV_CLIENT][entry.entry_id]
+    """Set up a OpenUV sensor based on a config entry."""
+    openuv = hass.data[DOMAIN][DATA_CLIENT][entry.entry_id]
 
     sensors = []
     for kind, attrs in SENSORS.items():
@@ -91,7 +88,7 @@ async def async_setup_entry(hass, entry, async_add_entities):
     async_add_entities(sensors, True)
 
 
-class OpenUvSensor(OpenUvEntity):
+class OpenUvSensor(OpenUvEntity, SensorEntity):
     """Define a binary sensor for OpenUV."""
 
     def __init__(self, openuv, sensor_type, name, icon, unit, entry_id):
