@@ -1,7 +1,4 @@
 """Webhooks used by rachio."""
-
-import logging
-
 from aiohttp import web
 
 from homeassistant.const import URL_API
@@ -16,6 +13,7 @@ from .const import (
     KEY_TYPE,
     SIGNAL_RACHIO_CONTROLLER_UPDATE,
     SIGNAL_RACHIO_RAIN_DELAY_UPDATE,
+    SIGNAL_RACHIO_RAIN_SENSOR_UPDATE,
     SIGNAL_RACHIO_SCHEDULE_UPDATE,
     SIGNAL_RACHIO_ZONE_UPDATE,
 )
@@ -29,14 +27,17 @@ SUBTYPE_COLD_REBOOT = "COLD_REBOOT"
 SUBTYPE_SLEEP_MODE_ON = "SLEEP_MODE_ON"
 SUBTYPE_SLEEP_MODE_OFF = "SLEEP_MODE_OFF"
 SUBTYPE_BROWNOUT_VALVE = "BROWNOUT_VALVE"
-SUBTYPE_RAIN_SENSOR_DETECTION_ON = "RAIN_SENSOR_DETECTION_ON"
-SUBTYPE_RAIN_SENSOR_DETECTION_OFF = "RAIN_SENSOR_DETECTION_OFF"
 
 # Rain delay values
 TYPE_RAIN_DELAY_STATUS = "RAIN_DELAY"
 SUBTYPE_RAIN_DELAY_ON = "RAIN_DELAY_ON"
 SUBTYPE_RAIN_DELAY_OFF = "RAIN_DELAY_OFF"
 
+# Rain sensor values
+TYPE_RAIN_SENSOR_STATUS = "RAIN_SENSOR_DETECTION"
+SUBTYPE_RAIN_SENSOR_DETECTION_ON = "RAIN_SENSOR_DETECTION_ON"
+SUBTYPE_RAIN_SENSOR_DETECTION_OFF = "RAIN_SENSOR_DETECTION_OFF"
+
 # Schedule webhook values
 TYPE_SCHEDULE_STATUS = "SCHEDULE_STATUS"
 SUBTYPE_SCHEDULE_STARTED = "SCHEDULE_STARTED"
@@ -54,12 +55,14 @@ SUBTYPE_ZONE_STOPPED = "ZONE_STOPPED"
 SUBTYPE_ZONE_COMPLETED = "ZONE_COMPLETED"
 SUBTYPE_ZONE_CYCLING = "ZONE_CYCLING"
 SUBTYPE_ZONE_CYCLING_COMPLETED = "ZONE_CYCLING_COMPLETED"
+SUBTYPE_ZONE_PAUSED = "ZONE_PAUSED"
 
 # Webhook callbacks
 LISTEN_EVENT_TYPES = [
     "DEVICE_STATUS_EVENT",
     "ZONE_STATUS_EVENT",
     "RAIN_DELAY_EVENT",
+    "RAIN_SENSOR_DETECTION_EVENT",
     "SCHEDULE_STATUS_EVENT",
 ]
 WEBHOOK_CONST_ID = "homeassistant.rachio:"
@@ -68,14 +71,12 @@ WEBHOOK_PATH = URL_API + DOMAIN
 SIGNAL_MAP = {
     TYPE_CONTROLLER_STATUS: SIGNAL_RACHIO_CONTROLLER_UPDATE,
     TYPE_RAIN_DELAY_STATUS: SIGNAL_RACHIO_RAIN_DELAY_UPDATE,
+    TYPE_RAIN_SENSOR_STATUS: SIGNAL_RACHIO_RAIN_SENSOR_UPDATE,
     TYPE_SCHEDULE_STATUS: SIGNAL_RACHIO_SCHEDULE_UPDATE,
     TYPE_ZONE_STATUS: SIGNAL_RACHIO_ZONE_UPDATE,
 }
 
 
-_LOGGER = logging.getLogger(__name__)
-
-
 @callback
 def async_register_webhook(hass, webhook_id, entry_id):
     """Register a webhook."""
