@@ -47,7 +47,7 @@ SENSORS = {
         name="Gas",
         device_class=DEVICE_CLASS_GAS,
         value=lambda value: value in ["mild", "heavy"],
-        device_state_attributes=lambda block: {"detected": block.gas},
+        extra_state_attributes=lambda block: {"detected": block.gas},
     ),
     ("sensor", "smoke"): BlockAttributeDescription(
         name="Smoke", device_class=DEVICE_CLASS_SMOKE
@@ -95,7 +95,7 @@ REST_SENSORS = {
         icon="mdi:update",
         value=lambda status, _: status["update"]["has_update"],
         default_enabled=False,
-        device_state_attributes=lambda status: {
+        extra_state_attributes=lambda status: {
             "latest_stable_version": status["update"]["new_version"],
             "installed_version": status["update"]["old_version"],
         },
