@@ -1,43 +1,49 @@
 """StarLine Account."""
+from __future__ import annotations
+
 from datetime import datetime, timedelta
-from typing import Any, Callable, Dict, Optional
+from typing import Any, Callable
 
 from starline import StarlineApi, StarlineDevice
 
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.event import async_track_time_interval
 
 from .const import (
+    _LOGGER,
     DATA_EXPIRES,
     DATA_SLID_TOKEN,
     DATA_SLNET_TOKEN,
     DATA_USER_ID,
     DEFAULT_SCAN_INTERVAL,
+    DEFAULT_SCAN_OBD_INTERVAL,
     DOMAIN,
-    LOGGER,
 )
 
 
 class StarlineAccount:
     """StarLine Account class."""
 
-    def __init__(self, hass: HomeAssistant, config_entry: ConfigEntry):
+    def __init__(self, hass: HomeAssistant, config_entry: ConfigEntry) -> None:
         """Initialize StarLine account."""
         self._hass: HomeAssistant = hass
         self._config_entry: ConfigEntry = config_entry
         self._update_interval: int = DEFAULT_SCAN_INTERVAL
-        self._unsubscribe_auto_updater: Optional[Callable] = None
+        self._update_obd_interval: int = DEFAULT_SCAN_OBD_INTERVAL
+        self._unsubscribe_auto_updater: Callable | None = None
+        self._unsubscribe_auto_obd_updater: Callable | None = None
         self._api: StarlineApi = StarlineApi(
             config_entry.data[DATA_USER_ID], config_entry.data[DATA_SLNET_TOKEN]
         )
 
-    def _check_slnet_token(self) -> None:
+    def _check_slnet_token(self, interval: int) -> None:
         """Check SLNet token expiration and update if needed."""
         now = datetime.now().timestamp()
         slnet_token_expires = self._config_entry.data[DATA_EXPIRES]
 
-        if now + self._update_interval > slnet_token_expires:
+        if now + interval > slnet_token_expires:
             self._update_slnet_token()
 
     def _update_slnet_token(self) -> None:
@@ -60,14 +66,18 @@ class StarlineAccount:
                 },
             )
         except Exception as err:  # pylint: disable=broad-except
-            LOGGER.error("Error updating SLNet token: %s", err)
-            pass
+            _LOGGER.error("Error updating SLNet token: %s", err)
 
     def _update_data(self):
         """Update StarLine data."""
-        self._check_slnet_token()
+        self._check_slnet_token(self._update_interval)
         self._api.update()
 
+    def _update_obd_data(self):
+        """Update StarLine OBD data."""
+        self._check_slnet_token(self._update_obd_interval)
+        self._api.update_obd()
+
     @property
     def api(self) -> StarlineApi:
         """Return the instance of the API."""
@@ -77,9 +87,13 @@ class StarlineAccount:
         """Update StarLine data."""
         await self._hass.async_add_executor_job(self._update_data)
 
+    async def update_obd(self, unused=None):
+        """Update StarLine OBD data."""
+        await self._hass.async_add_executor_job(self._update_obd_data)
+
     def set_update_interval(self, interval: int) -> None:
         """Set StarLine API update interval."""
-        LOGGER.debug("Setting update interval: %ds", interval)
+        _LOGGER.debug("Setting update interval: %ds", interval)
         self._update_interval = interval
         if self._unsubscribe_auto_updater is not None:
             self._unsubscribe_auto_updater()
@@ -89,15 +103,30 @@ class StarlineAccount:
             self._hass, self.update, delta
         )
 
+    def set_update_obd_interval(self, interval: int) -> None:
+        """Set StarLine API OBD update interval."""
+        _LOGGER.debug("Setting OBD update interval: %ds", interval)
+        self._update_obd_interval = interval
+        if self._unsubscribe_auto_obd_updater is not None:
+            self._unsubscribe_auto_obd_updater()
+
+        delta = timedelta(seconds=interval)
+        self._unsubscribe_auto_obd_updater = async_track_time_interval(
+            self._hass, self.update_obd, delta
+        )
+
     def unload(self):
         """Unload StarLine API."""
-        LOGGER.debug("Unloading StarLine API.")
+        _LOGGER.debug("Unloading StarLine API")
         if self._unsubscribe_auto_updater is not None:
             self._unsubscribe_auto_updater()
             self._unsubscribe_auto_updater = None
+        if self._unsubscribe_auto_obd_updater is not None:
+            self._unsubscribe_auto_obd_updater()
+            self._unsubscribe_auto_obd_updater = None
 
     @staticmethod
-    def device_info(device: StarlineDevice) -> Dict[str, Any]:
+    def device_info(device: StarlineDevice) -> DeviceInfo:
         """Device information for entities."""
         return {
             "identifiers": {(DOMAIN, device.device_id)},
@@ -108,7 +137,7 @@ class StarlineAccount:
         }
 
     @staticmethod
-    def gps_attrs(device: StarlineDevice) -> Dict[str, Any]:
+    def gps_attrs(device: StarlineDevice) -> dict[str, Any]:
         """Attributes for device tracker."""
         return {
             "updated": datetime.utcfromtimestamp(device.position["ts"]).isoformat(),
@@ -116,7 +145,7 @@ class StarlineAccount:
         }
 
     @staticmethod
-    def balance_attrs(device: StarlineDevice) -> Dict[str, Any]:
+    def balance_attrs(device: StarlineDevice) -> dict[str, Any]:
         """Attributes for balance sensor."""
         return {
             "operator": device.balance.get("operator"),
@@ -125,7 +154,7 @@ class StarlineAccount:
         }
 
     @staticmethod
-    def gsm_attrs(device: StarlineDevice) -> Dict[str, Any]:
+    def gsm_attrs(device: StarlineDevice) -> dict[str, Any]:
         """Attributes for GSM sensor."""
         return {
             "raw": device.gsm_level,
@@ -135,9 +164,14 @@ class StarlineAccount:
         }
 
     @staticmethod
-    def engine_attrs(device: StarlineDevice) -> Dict[str, Any]:
+    def engine_attrs(device: StarlineDevice) -> dict[str, Any]:
         """Attributes for engine switch."""
         return {
             "autostart": device.car_state.get("r_start"),
             "ignition": device.car_state.get("run"),
         }
+
+    @staticmethod
+    def errors_attrs(device: StarlineDevice) -> dict[str, Any]:
+        """Attributes for errors sensor."""
+        return {"errors": device.errors.get("errors")}
