@@ -1,10 +1,14 @@
 """Provide functionality to record stream."""
+from __future__ import annotations
+
+from collections import deque
+from io import BytesIO
 import logging
 import os
 import threading
-from typing import Deque, List
 
 import av
+from av.container import OutputContainer
 
 from homeassistant.core import HomeAssistant, callback
 
@@ -19,13 +23,18 @@ def async_setup_recorder(hass):
     """Only here so Provider Registry works."""
 
 
-def recorder_save_worker(file_out: str, segments: Deque[Segment]):
+def recorder_save_worker(file_out: str, segments: deque[Segment]):
     """Handle saving stream."""
+
+    if not segments:
+        _LOGGER.error("Recording failed to capture anything")
+        return
+
     if not os.path.exists(os.path.dirname(file_out)):
         os.makedirs(os.path.dirname(file_out), exist_ok=True)
 
-    pts_adjuster = {"video": None, "audio": None}
-    output = None
+    pts_adjuster: dict[str, int | None] = {"video": None, "audio": None}
+    output: OutputContainer | None = None
     output_v = None
     output_a = None
 
@@ -43,7 +52,11 @@ def recorder_save_worker(file_out: str, segments: Deque[Segment]):
         last_sequence = segment.sequence
 
         # Open segment
-        source = av.open(segment.segment, "r", format=SEGMENT_CONTAINER_FORMAT)
+        source = av.open(
+            BytesIO(segment.init + segment.moof_data),
+            "r",
+            format=SEGMENT_CONTAINER_FORMAT,
+        )
         source_v = source.streams.video[0]
         source_a = source.streams.audio[0] if len(source.streams.audio) > 0 else None
 
@@ -93,7 +106,8 @@ def recorder_save_worker(file_out: str, segments: Deque[Segment]):
 
         source.close()
 
-    output.close()
+    if output is not None:
+        output.close()
 
 
 @PROVIDERS.register("recorder")
@@ -110,7 +124,7 @@ class RecorderOutput(StreamOutput):
         """Return provider name."""
         return "recorder"
 
-    def prepend(self, segments: List[Segment]) -> None:
+    def prepend(self, segments: list[Segment]) -> None:
         """Prepend segments to existing list."""
         self._segments.extendleft(reversed(segments))
 
