@@ -3,9 +3,6 @@ import asyncio
 from datetime import timedelta
 import logging
 
-import async_timeout
-from pywemo.ouimeaux_device.api.service import ActionException
-
 from homeassistant import util
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
@@ -22,6 +19,7 @@ from homeassistant.helpers.dispatcher import async_dispatcher_connect
 import homeassistant.util.color as color_util
 
 from .const import DOMAIN as WEMO_DOMAIN
+from .entity import WemoEntity, WemoSubscriptionEntity
 
 MIN_TIME_BETWEEN_SCANS = timedelta(seconds=10)
 MIN_TIME_BETWEEN_FORCED_SCANS = timedelta(milliseconds=100)
@@ -32,6 +30,10 @@ SUPPORT_WEMO = (
     SUPPORT_BRIGHTNESS | SUPPORT_COLOR_TEMP | SUPPORT_COLOR | SUPPORT_TRANSITION
 )
 
+# The WEMO_ constants below come from pywemo itself
+WEMO_ON = 1
+WEMO_OFF = 0
+
 
 async def async_setup_entry(hass, config_entry, async_add_entities):
     """Set up WeMo lights."""
@@ -77,21 +79,17 @@ def setup_bridge(hass, bridge, async_add_entities):
     update_lights()
 
 
-class WemoLight(LightEntity):
+class WemoLight(WemoEntity, LightEntity):
     """Representation of a WeMo light."""
 
     def __init__(self, device, update_lights):
         """Initialize the WeMo light."""
-        self.wemo = device
-        self._state = None
+        super().__init__(device)
         self._update_lights = update_lights
-        self._available = True
-        self._update_lock = None
         self._brightness = None
         self._hs_color = None
         self._color_temp = None
         self._is_on = None
-        self._name = self.wemo.name
         self._unique_id = self.wemo.uniqueID
         self._model_name = type(self.wemo).__name__
 
@@ -103,18 +101,13 @@ class WemoLight(LightEntity):
     @property
     def unique_id(self):
         """Return the ID of this light."""
-        return self._unique_id
-
-    @property
-    def name(self):
-        """Return the name of the light."""
-        return self._name
+        return self.wemo.uniqueID
 
     @property
     def device_info(self):
         """Return the device info."""
         return {
-            "name": self._name,
+            "name": self.name,
             "identifiers": {(WEMO_DOMAIN, self._unique_id)},
             "model": self._model_name,
             "manufacturer": "Belkin",
@@ -145,11 +138,6 @@ class WemoLight(LightEntity):
         """Flag supported features."""
         return SUPPORT_WEMO
 
-    @property
-    def available(self):
-        """Return if light is available."""
-        return self._available
-
     def turn_on(self, **kwargs):
         """Turn the light on."""
         xy_color = None
@@ -168,42 +156,37 @@ class WemoLight(LightEntity):
             "force_update": False,
         }
 
-        try:
+        with self._wemo_exception_handler("turn on"):
             if xy_color is not None:
                 self.wemo.set_color(xy_color, transition=transition_time)
 
             if color_temp is not None:
                 self.wemo.set_temperature(mireds=color_temp, transition=transition_time)
 
-            self.wemo.turn_on(**turn_on_kwargs)
-        except ActionException as err:
-            _LOGGER.warning("Error while turning on device %s (%s)", self.name, err)
-            self._available = False
+            if self.wemo.turn_on(**turn_on_kwargs):
+                self._state["onoff"] = WEMO_ON
+
+        self.schedule_update_ha_state()
 
     def turn_off(self, **kwargs):
         """Turn the light off."""
         transition_time = int(kwargs.get(ATTR_TRANSITION, 0))
 
-        try:
-            self.wemo.turn_off(transition=transition_time)
-        except ActionException as err:
-            _LOGGER.warning("Error while turning off device %s (%s)", self.name, err)
-            self._available = False
+        with self._wemo_exception_handler("turn off"):
+            if self.wemo.turn_off(transition=transition_time):
+                self._state["onoff"] = WEMO_OFF
+
+        self.schedule_update_ha_state()
 
     def _update(self, force_update=True):
         """Synchronize state with bridge."""
-        try:
+        with self._wemo_exception_handler("update status") as handler:
             self._update_lights(no_throttle=force_update)
             self._state = self.wemo.state
-        except (AttributeError, ActionException) as err:
-            _LOGGER.warning("Could not update status for %s (%s)", self.name, err)
-            self._available = False
-            self.wemo.reconnect_with_device()
-        else:
-            self._is_on = self._state.get("onoff") != 0
+        if handler.success:
+            self._is_on = self._state.get("onoff") != WEMO_OFF
             self._brightness = self._state.get("level", 255)
             self._color_temp = self._state.get("temperature_mireds")
-            self._available = True
 
             xy_color = self._state.get("color_xy")
 
@@ -212,106 +195,14 @@ class WemoLight(LightEntity):
             else:
                 self._hs_color = None
 
-    async def async_update(self):
-        """Synchronize state with bridge."""
-        # If an update is in progress, we don't do anything
-        if self._update_lock.locked():
-            return
 
-        try:
-            with async_timeout.timeout(5):
-                await asyncio.shield(self._async_locked_update(True))
-        except asyncio.TimeoutError:
-            _LOGGER.warning("Lost connection to %s", self.name)
-            self._available = False
-
-    async def _async_locked_update(self, force_update):
-        """Try updating within an async lock."""
-        async with self._update_lock:
-            await self.hass.async_add_executor_job(self._update, force_update)
-
-
-class WemoDimmer(LightEntity):
+class WemoDimmer(WemoSubscriptionEntity, LightEntity):
     """Representation of a WeMo dimmer."""
 
     def __init__(self, device):
         """Initialize the WeMo dimmer."""
-        self.wemo = device
-        self._state = None
-        self._available = True
-        self._update_lock = None
+        super().__init__(device)
         self._brightness = None
-        self._model_name = self.wemo.model_name
-        self._name = self.wemo.name
-        self._serialnumber = self.wemo.serialnumber
-
-    def _subscription_callback(self, _device, _type, _params):
-        """Update the state by the Wemo device."""
-        _LOGGER.debug("Subscription update for %s", self.name)
-        updated = self.wemo.subscription_update(_type, _params)
-        self.hass.add_job(self._async_locked_subscription_callback(not updated))
-
-    async def _async_locked_subscription_callback(self, force_update):
-        """Handle an update from a subscription."""
-        # If an update is in progress, we don't do anything
-        if self._update_lock.locked():
-            return
-
-        await self._async_locked_update(force_update)
-        self.async_write_ha_state()
-
-    async def async_added_to_hass(self):
-        """Wemo dimmer added to Home Assistant."""
-        # Define inside async context so we know our event loop
-        self._update_lock = asyncio.Lock()
-
-        registry = self.hass.data[WEMO_DOMAIN]["registry"]
-        await self.hass.async_add_executor_job(registry.register, self.wemo)
-        registry.on(self.wemo, None, self._subscription_callback)
-
-    async def async_update(self):
-        """Update WeMo state.
-
-        Wemo has an aggressive retry logic that sometimes can take over a
-        minute to return. If we don't get a state after 5 seconds, assume the
-        Wemo dimmer is unreachable. If update goes through, it will be made
-        available again.
-        """
-        # If an update is in progress, we don't do anything
-        if self._update_lock.locked():
-            return
-
-        try:
-            with async_timeout.timeout(5):
-                await asyncio.shield(self._async_locked_update(True))
-        except asyncio.TimeoutError:
-            _LOGGER.warning("Lost connection to %s", self.name)
-            self._available = False
-
-    async def _async_locked_update(self, force_update):
-        """Try updating within an async lock."""
-        async with self._update_lock:
-            await self.hass.async_add_executor_job(self._update, force_update)
-
-    @property
-    def unique_id(self):
-        """Return the ID of this WeMo dimmer."""
-        return self._serialnumber
-
-    @property
-    def name(self):
-        """Return the name of the dimmer if any."""
-        return self._name
-
-    @property
-    def device_info(self):
-        """Return the device info."""
-        return {
-            "name": self._name,
-            "identifiers": {(WEMO_DOMAIN, self._serialnumber)},
-            "model": self._model_name,
-            "manufacturer": "Belkin",
-        }
 
     @property
     def supported_features(self):
@@ -323,27 +214,14 @@ class WemoDimmer(LightEntity):
         """Return the brightness of this light between 1 and 100."""
         return self._brightness
 
-    @property
-    def is_on(self):
-        """Return true if dimmer is on. Standby is on."""
-        return self._state
-
     def _update(self, force_update=True):
         """Update the device state."""
-        try:
+        with self._wemo_exception_handler("update status"):
             self._state = self.wemo.get_state(force_update)
 
             wemobrightness = int(self.wemo.get_brightness(force_update))
             self._brightness = int((wemobrightness * 255) / 100)
 
-            if not self._available:
-                _LOGGER.info("Reconnected to %s", self.name)
-                self._available = True
-        except (AttributeError, ActionException) as err:
-            _LOGGER.warning("Could not update status for %s (%s)", self.name, err)
-            self._available = False
-            self.wemo.reconnect_with_device()
-
     def turn_on(self, **kwargs):
         """Turn the dimmer on."""
         # Wemo dimmer switches use a range of [0, 100] to control
@@ -354,22 +232,18 @@ class WemoDimmer(LightEntity):
         else:
             brightness = 255
 
-        try:
-            self.wemo.on()
+        with self._wemo_exception_handler("turn on"):
+            if self.wemo.on():
+                self._state = WEMO_ON
+
             self.wemo.set_brightness(brightness)
-        except ActionException as err:
-            _LOGGER.warning("Error while turning on device %s (%s)", self.name, err)
-            self._available = False
+
+        self.schedule_update_ha_state()
 
     def turn_off(self, **kwargs):
         """Turn the dimmer off."""
-        try:
-            self.wemo.off()
-        except ActionException as err:
-            _LOGGER.warning("Error while turning on device %s (%s)", self.name, err)
-            self._available = False
+        with self._wemo_exception_handler("turn off"):
+            if self.wemo.off():
+                self._state = WEMO_OFF
 
-    @property
-    def available(self):
-        """Return if dimmer is available."""
-        return self._available
+        self.schedule_update_ha_state()
