@@ -2,9 +2,8 @@
 from dataclasses import dataclass
 import logging
 
-from miio import AirQualityMonitor  # pylint: disable=import-error
-from miio import DeviceException
-from miio.gateway import (
+from miio import AirQualityMonitor, DeviceException
+from miio.gateway.gateway import (
     GATEWAY_MODEL_AC_V1,
     GATEWAY_MODEL_AC_V2,
     GATEWAY_MODEL_AC_V3,
@@ -13,30 +12,36 @@ from miio.gateway import (
 )
 import voluptuous as vol
 
-from homeassistant.components.sensor import PLATFORM_SCHEMA
+from homeassistant.components.sensor import (
+    PLATFORM_SCHEMA,
+    STATE_CLASS_MEASUREMENT,
+    SensorEntity,
+)
+from homeassistant.config_entries import SOURCE_IMPORT
 from homeassistant.const import (
+    ATTR_BATTERY_LEVEL,
     CONF_HOST,
     CONF_NAME,
     CONF_TOKEN,
     DEVICE_CLASS_HUMIDITY,
     DEVICE_CLASS_ILLUMINANCE,
+    DEVICE_CLASS_POWER,
     DEVICE_CLASS_PRESSURE,
     DEVICE_CLASS_TEMPERATURE,
     PERCENTAGE,
+    POWER_WATT,
     PRESSURE_HPA,
     TEMP_CELSIUS,
 )
-from homeassistant.exceptions import PlatformNotReady
 import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.entity import Entity
 
-from .const import CONF_FLOW_TYPE, CONF_GATEWAY, DOMAIN, KEY_COORDINATOR
+from .const import CONF_DEVICE, CONF_FLOW_TYPE, CONF_GATEWAY, DOMAIN, KEY_COORDINATOR
+from .device import XiaomiMiioEntity
 from .gateway import XiaomiGatewayDevice
 
 _LOGGER = logging.getLogger(__name__)
 
 DEFAULT_NAME = "Xiaomi Miio Sensor"
-DATA_KEY = "sensor.xiaomi_miio"
 UNIT_LUMEN = "lm"
 
 PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
@@ -49,13 +54,11 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 
 ATTR_POWER = "power"
 ATTR_CHARGING = "charging"
-ATTR_BATTERY_LEVEL = "battery_level"
 ATTR_DISPLAY_CLOCK = "display_clock"
 ATTR_NIGHT_MODE = "night_mode"
 ATTR_NIGHT_TIME_BEGIN = "night_time_begin"
 ATTR_NIGHT_TIME_END = "night_time_end"
 ATTR_SENSOR_STATE = "sensor_state"
-ATTR_MODEL = "model"
 
 SUCCESS = ["ok"]
 
@@ -67,21 +70,49 @@ class SensorType:
     unit: str = None
     icon: str = None
     device_class: str = None
+    state_class: str = None
 
 
 GATEWAY_SENSOR_TYPES = {
     "temperature": SensorType(
-        unit=TEMP_CELSIUS, icon=None, device_class=DEVICE_CLASS_TEMPERATURE
+        unit=TEMP_CELSIUS,
+        icon=None,
+        device_class=DEVICE_CLASS_TEMPERATURE,
+        state_class=STATE_CLASS_MEASUREMENT,
     ),
     "humidity": SensorType(
-        unit=PERCENTAGE, icon=None, device_class=DEVICE_CLASS_HUMIDITY
+        unit=PERCENTAGE,
+        icon=None,
+        device_class=DEVICE_CLASS_HUMIDITY,
+        state_class=STATE_CLASS_MEASUREMENT,
     ),
     "pressure": SensorType(
-        unit=PRESSURE_HPA, icon=None, device_class=DEVICE_CLASS_PRESSURE
+        unit=PRESSURE_HPA,
+        icon=None,
+        device_class=DEVICE_CLASS_PRESSURE,
+        state_class=STATE_CLASS_MEASUREMENT,
+    ),
+    "load_power": SensorType(
+        unit=POWER_WATT, icon=None, device_class=DEVICE_CLASS_POWER
     ),
 }
 
 
+async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
+    """Import Miio configuration from YAML."""
+    _LOGGER.warning(
+        "Loading Xiaomi Miio Sensor via platform setup is deprecated. "
+        "Please remove it from your configuration"
+    )
+    hass.async_create_task(
+        hass.config_entries.flow.async_init(
+            DOMAIN,
+            context={"source": SOURCE_IMPORT},
+            data=config,
+        )
+    )
+
+
 async def async_setup_entry(hass, config_entry, async_add_entities):
     """Set up the Xiaomi sensor from a config entry."""
     entities = []
@@ -115,48 +146,26 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
                     ]
                 )
 
+    if config_entry.data[CONF_FLOW_TYPE] == CONF_DEVICE:
+        host = config_entry.data[CONF_HOST]
+        token = config_entry.data[CONF_TOKEN]
+        name = config_entry.title
+        unique_id = config_entry.unique_id
+
+        _LOGGER.debug("Initializing with host %s (token %s...)", host, token[:5])
+
+        device = AirQualityMonitor(host, token)
+        entities.append(XiaomiAirQualityMonitor(name, device, config_entry, unique_id))
+
     async_add_entities(entities, update_before_add=True)
 
 
-async def async_setup_platform(hass, config, async_add_entities, discovery_info=None):
-    """Set up the sensor from config."""
-    if DATA_KEY not in hass.data:
-        hass.data[DATA_KEY] = {}
-
-    host = config[CONF_HOST]
-    token = config[CONF_TOKEN]
-    name = config[CONF_NAME]
-
-    _LOGGER.info("Initializing with host %s (token %s...)", host, token[:5])
-
-    try:
-        air_quality_monitor = AirQualityMonitor(host, token)
-        device_info = await hass.async_add_executor_job(air_quality_monitor.info)
-        model = device_info.model
-        unique_id = f"{model}-{device_info.mac_address}"
-        _LOGGER.info(
-            "%s %s %s detected",
-            model,
-            device_info.firmware_version,
-            device_info.hardware_version,
-        )
-        device = XiaomiAirQualityMonitor(name, air_quality_monitor, model, unique_id)
-    except DeviceException as ex:
-        raise PlatformNotReady from ex
-
-    hass.data[DATA_KEY][host] = device
-    async_add_entities([device], update_before_add=True)
-
-
-class XiaomiAirQualityMonitor(Entity):
+class XiaomiAirQualityMonitor(XiaomiMiioEntity, SensorEntity):
     """Representation of a Xiaomi Air Quality Monitor."""
 
-    def __init__(self, name, device, model, unique_id):
+    def __init__(self, name, device, entry, unique_id):
         """Initialize the entity."""
-        self._name = name
-        self._device = device
-        self._model = model
-        self._unique_id = unique_id
+        super().__init__(name, device, entry, unique_id)
 
         self._icon = "mdi:cloud"
         self._unit_of_measurement = "AQI"
@@ -171,19 +180,8 @@ class XiaomiAirQualityMonitor(Entity):
             ATTR_NIGHT_TIME_BEGIN: None,
             ATTR_NIGHT_TIME_END: None,
             ATTR_SENSOR_STATE: None,
-            ATTR_MODEL: self._model,
         }
 
-    @property
-    def unique_id(self):
-        """Return an unique ID."""
-        return self._unique_id
-
-    @property
-    def name(self):
-        """Return the name of this entity, if any."""
-        return self._name
-
     @property
     def unit_of_measurement(self):
         """Return the unit of measurement of this entity, if any."""
@@ -205,7 +203,7 @@ class XiaomiAirQualityMonitor(Entity):
         return self._state
 
     @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return the state attributes of the device."""
         return self._state_attrs
 
@@ -236,7 +234,7 @@ class XiaomiAirQualityMonitor(Entity):
                 _LOGGER.error("Got exception while fetching the state: %s", ex)
 
 
-class XiaomiGatewaySensor(XiaomiGatewayDevice):
+class XiaomiGatewaySensor(XiaomiGatewayDevice, SensorEntity):
     """Representation of a XiaomiGatewaySensor."""
 
     def __init__(self, coordinator, sub_device, entry, data_key):
@@ -261,13 +259,18 @@ class XiaomiGatewaySensor(XiaomiGatewayDevice):
         """Return the device class of this entity."""
         return GATEWAY_SENSOR_TYPES[self._data_key].device_class
 
+    @property
+    def state_class(self):
+        """Return the state class of this entity."""
+        return GATEWAY_SENSOR_TYPES[self._data_key].state_class
+
     @property
     def state(self):
         """Return the state of the sensor."""
         return self._sub_device.status[self._data_key]
 
 
-class XiaomiGatewayIlluminanceSensor(Entity):
+class XiaomiGatewayIlluminanceSensor(SensorEntity):
     """Representation of the gateway device's illuminance sensor."""
 
     def __init__(self, gateway_device, gateway_name, gateway_device_id):
