@@ -2,28 +2,50 @@
 from __future__ import annotations
 
 import asyncio
+from collections.abc import Coroutine, Iterable
 from contextvars import ContextVar
 from datetime import datetime, timedelta
+import logging
 from logging import Logger
 from types import ModuleType
-from typing import TYPE_CHECKING, Callable, Coroutine, Dict, Iterable, List, Optional
+from typing import TYPE_CHECKING, Any, Callable
+
+from typing_extensions import Protocol
+import voluptuous as vol
 
 from homeassistant import config_entries
-from homeassistant.const import ATTR_RESTORED, DEVICE_DEFAULT_NAME
+from homeassistant.const import (
+    ATTR_RESTORED,
+    DEVICE_DEFAULT_NAME,
+    EVENT_HOMEASSISTANT_STARTED,
+)
 from homeassistant.core import (
     CALLBACK_TYPE,
+    CoreState,
+    HomeAssistant,
     ServiceCall,
     callback,
     split_entity_id,
     valid_entity_id,
 )
-from homeassistant.exceptions import HomeAssistantError, PlatformNotReady
-from homeassistant.helpers import config_validation as cv, service
-from homeassistant.helpers.typing import HomeAssistantType
+from homeassistant.exceptions import (
+    HomeAssistantError,
+    PlatformNotReady,
+    RequiredParameterMissing,
+)
+from homeassistant.setup import async_start_setup
 from homeassistant.util.async_ import run_callback_threadsafe
 
-from .entity_registry import DISABLED_INTEGRATION
+from . import (
+    config_validation as cv,
+    device_registry as dev_reg,
+    entity_registry as ent_reg,
+    service,
+)
+from .device_registry import DeviceRegistry
+from .entity_registry import DISABLED_INTEGRATION, EntityRegistry
 from .event import async_call_later, async_track_time_interval
+from .typing import ConfigType, DiscoveryInfoType
 
 if TYPE_CHECKING:
     from .entity import Entity
@@ -38,6 +60,17 @@ PLATFORM_NOT_READY_RETRIES = 10
 DATA_ENTITY_PLATFORM = "entity_platform"
 PLATFORM_NOT_READY_BASE_WAIT_TIME = 30  # seconds
 
+_LOGGER = logging.getLogger(__name__)
+
+
+class AddEntitiesCallback(Protocol):
+    """Protocol type for EntityPlatform.add_entities callback."""
+
+    def __call__(
+        self, new_entities: Iterable[Entity], update_before_add: bool = False
+    ) -> None:
+        """Define add_entities type."""
+
 
 class EntityPlatform:
     """Manage the entities for a single platform."""
@@ -45,14 +78,14 @@ class EntityPlatform:
     def __init__(
         self,
         *,
-        hass: HomeAssistantType,
+        hass: HomeAssistant,
         logger: Logger,
         domain: str,
         platform_name: str,
-        platform: Optional[ModuleType],
+        platform: ModuleType | None,
         scan_interval: timedelta,
-        entity_namespace: Optional[str],
-    ):
+        entity_namespace: str | None,
+    ) -> None:
         """Initialize the entity platform."""
         self.hass = hass
         self.logger = logger
@@ -61,18 +94,18 @@ class EntityPlatform:
         self.platform = platform
         self.scan_interval = scan_interval
         self.entity_namespace = entity_namespace
-        self.config_entry: Optional[config_entries.ConfigEntry] = None
-        self.entities: Dict[str, Entity] = {}  # pylint: disable=used-before-assignment
-        self._tasks: List[asyncio.Future] = []
+        self.config_entry: config_entries.ConfigEntry | None = None
+        self.entities: dict[str, Entity] = {}
+        self._tasks: list[asyncio.Future] = []
         # Stop tracking tasks after setup is completed
         self._setup_complete = False
         # Method to cancel the state change listener
-        self._async_unsub_polling: Optional[CALLBACK_TYPE] = None
+        self._async_unsub_polling: CALLBACK_TYPE | None = None
         # Method to cancel the retry of setup
-        self._async_cancel_retry_setup: Optional[CALLBACK_TYPE] = None
-        self._process_updates: Optional[asyncio.Lock] = None
+        self._async_cancel_retry_setup: CALLBACK_TYPE | None = None
+        self._process_updates: asyncio.Lock | None = None
 
-        self.parallel_updates: Optional[asyncio.Semaphore] = None
+        self.parallel_updates: asyncio.Semaphore | None = None
 
         # Platform is None for the EntityComponent "catch-all" EntityPlatform
         # which powers entity_component.add_entities
@@ -89,7 +122,7 @@ class EntityPlatform:
     @callback
     def _get_parallel_updates_semaphore(
         self, entity_has_async_update: bool
-    ) -> Optional[asyncio.Semaphore]:
+    ) -> asyncio.Semaphore | None:
         """Get or create a semaphore for parallel updates.
 
         Semaphore will be created on demand because we base it off if update method is async or not.
@@ -117,7 +150,11 @@ class EntityPlatform:
 
         return self.parallel_updates
 
-    async def async_setup(self, platform_config, discovery_info=None):  # type: ignore[no-untyped-def]
+    async def async_setup(
+        self,
+        platform_config: ConfigType,
+        discovery_info: DiscoveryInfoType | None = None,
+    ) -> None:
         """Set up the platform from a config file."""
         platform = self.platform
         hass = self.hass
@@ -156,6 +193,18 @@ class EntityPlatform:
 
         await self._async_setup_platform(async_create_setup_task)
 
+    async def async_shutdown(self) -> None:
+        """Call when Home Assistant is stopping."""
+        self.async_cancel_retry_setup()
+        self.async_unsub_polling()
+
+    @callback
+    def async_cancel_retry_setup(self) -> None:
+        """Cancel retry setup."""
+        if self._async_cancel_retry_setup is not None:
+            self._async_cancel_retry_setup()
+            self._async_cancel_retry_setup = None
+
     async def async_setup_entry(self, config_entry: config_entries.ConfigEntry) -> bool:
         """Set up the platform from a config entry."""
         # Store it so that we can save config entry ID in entity registry
@@ -163,9 +212,10 @@ class EntityPlatform:
         platform = self.platform
 
         @callback
-        def async_create_setup_task():  # type: ignore[no-untyped-def]
+        def async_create_setup_task() -> Coroutine:
             """Get task to set up platform."""
-            return platform.async_setup_entry(  # type: ignore
+            config_entries.current_entry.set(config_entry)
+            return platform.async_setup_entry(  # type: ignore[no-any-return,union-attr]
                 self.hass, config_entry, self._async_schedule_add_entities
             )
 
@@ -192,59 +242,75 @@ class EntityPlatform:
             self.platform_name,
             SLOW_SETUP_WARNING,
         )
+        with async_start_setup(hass, [full_name]):
+            try:
+                task = async_create_setup_task()
 
-        try:
-            task = async_create_setup_task()
+                async with hass.timeout.async_timeout(SLOW_SETUP_MAX_WAIT, self.domain):
+                    await asyncio.shield(task)
 
-            async with hass.timeout.async_timeout(SLOW_SETUP_MAX_WAIT, self.domain):
-                await asyncio.shield(task)
+                # Block till all entities are done
+                while self._tasks:
+                    pending = [task for task in self._tasks if not task.done()]
+                    self._tasks.clear()
 
-            # Block till all entities are done
-            while self._tasks:
-                pending = [task for task in self._tasks if not task.done()]
-                self._tasks.clear()
+                    if pending:
+                        await asyncio.gather(*pending)
 
-                if pending:
-                    await asyncio.gather(*pending)
+                hass.config.components.add(full_name)
+                self._setup_complete = True
+                return True
+            except PlatformNotReady as ex:
+                tries += 1
+                wait_time = min(tries, 6) * PLATFORM_NOT_READY_BASE_WAIT_TIME
+                message = str(ex)
+                ready_message = f"ready yet: {message}" if message else "ready yet"
+                if tries == 1:
+                    logger.warning(
+                        "Platform %s not %s; Retrying in background in %d seconds",
+                        self.platform_name,
+                        ready_message,
+                        wait_time,
+                    )
+                else:
+                    logger.debug(
+                        "Platform %s not %s; Retrying in %d seconds",
+                        self.platform_name,
+                        ready_message,
+                        wait_time,
+                    )
 
-            hass.config.components.add(full_name)
-            self._setup_complete = True
-            return True
-        except PlatformNotReady:
-            tries += 1
-            wait_time = min(tries, 6) * PLATFORM_NOT_READY_BASE_WAIT_TIME
-            logger.warning(
-                "Platform %s not ready yet. Retrying in %d seconds.",
-                self.platform_name,
-                wait_time,
-            )
+                async def setup_again(*_args: Any) -> None:
+                    """Run setup again."""
+                    self._async_cancel_retry_setup = None
+                    await self._async_setup_platform(async_create_setup_task, tries)
 
-            async def setup_again(now):  # type: ignore[no-untyped-def]
-                """Run setup again."""
-                self._async_cancel_retry_setup = None
-                await self._async_setup_platform(async_create_setup_task, tries)
-
-            self._async_cancel_retry_setup = async_call_later(
-                hass, wait_time, setup_again
-            )
-            return False
-        except asyncio.TimeoutError:
-            logger.error(
-                "Setup of platform %s is taking longer than %s seconds."
-                " Startup will proceed without waiting any longer.",
-                self.platform_name,
-                SLOW_SETUP_MAX_WAIT,
-            )
-            return False
-        except Exception:  # pylint: disable=broad-except
-            logger.exception(
-                "Error while setting up %s platform for %s",
-                self.platform_name,
-                self.domain,
-            )
-            return False
-        finally:
-            warn_task.cancel()
+                if hass.state == CoreState.running:
+                    self._async_cancel_retry_setup = async_call_later(
+                        hass, wait_time, setup_again
+                    )
+                else:
+                    self._async_cancel_retry_setup = hass.bus.async_listen_once(
+                        EVENT_HOMEASSISTANT_STARTED, setup_again
+                    )
+                return False
+            except asyncio.TimeoutError:
+                logger.error(
+                    "Setup of platform %s is taking longer than %s seconds."
+                    " Startup will proceed without waiting any longer.",
+                    self.platform_name,
+                    SLOW_SETUP_MAX_WAIT,
+                )
+                return False
+            except Exception:  # pylint: disable=broad-except
+                logger.exception(
+                    "Error while setting up %s platform for %s",
+                    self.platform_name,
+                    self.domain,
+                )
+                return False
+            finally:
+                warn_task.cancel()
 
     def _schedule_add_entities(
         self, new_entities: Iterable[Entity], update_before_add: bool = False
@@ -298,10 +364,10 @@ class EntityPlatform:
 
         hass = self.hass
 
-        device_registry = await hass.helpers.device_registry.async_get_registry()
-        entity_registry = await hass.helpers.entity_registry.async_get_registry()
+        device_registry = dev_reg.async_get(hass)
+        entity_registry = ent_reg.async_get(hass)
         tasks = [
-            self._async_add_entity(  # type: ignore
+            self._async_add_entity(
                 entity, update_before_add, entity_registry, device_registry
             )
             for entity in new_entities
@@ -330,8 +396,10 @@ class EntityPlatform:
             )
             raise
 
-        if self._async_unsub_polling is not None or not any(
-            entity.should_poll for entity in self.entities.values()
+        if (
+            (self.config_entry and self.config_entry.pref_disable_polling)
+            or self._async_unsub_polling is not None
+            or not any(entity.should_poll for entity in self.entities.values())
         ):
             return
 
@@ -341,9 +409,13 @@ class EntityPlatform:
             self.scan_interval,
         )
 
-    async def _async_add_entity(  # type: ignore[no-untyped-def]
-        self, entity, update_before_add, entity_registry, device_registry
-    ):
+    async def _async_add_entity(  # noqa: C901
+        self,
+        entity: Entity,
+        update_before_add: bool,
+        entity_registry: EntityRegistry,
+        device_registry: DeviceRegistry,
+    ) -> None:
         """Add an entity to the platform."""
         if entity is None:
             raise ValueError("Entity cannot be None")
@@ -364,7 +436,8 @@ class EntityPlatform:
                 return
 
         requested_entity_id = None
-        suggested_object_id: Optional[str] = None
+        suggested_object_id: str | None = None
+        generate_new_entity_id = False
 
         # Get entity_id from unique ID registration
         if entity.unique_id is not None:
@@ -372,13 +445,13 @@ class EntityPlatform:
                 requested_entity_id = entity.entity_id
                 suggested_object_id = split_entity_id(entity.entity_id)[1]
             else:
-                suggested_object_id = entity.name
+                suggested_object_id = entity.name  # type: ignore[unreachable]
 
             if self.entity_namespace is not None:
                 suggested_object_id = f"{self.entity_namespace} {suggested_object_id}"
 
             if self.config_entry is not None:
-                config_entry_id: Optional[str] = self.config_entry.entry_id
+                config_entry_id: str | None = self.config_entry.entry_id
             else:
                 config_entry_id = None
 
@@ -402,13 +475,15 @@ class EntityPlatform:
                     "suggested_area",
                 ):
                     if key in device_info:
-                        processed_dev_info[key] = device_info[key]
+                        processed_dev_info[key] = device_info[key]  # type: ignore[misc]
 
-                device = device_registry.async_get_or_create(**processed_dev_info)
-                if device:
+                try:
+                    device = device_registry.async_get_or_create(**processed_dev_info)  # type: ignore[arg-type]
                     device_id = device.id
+                except RequiredParameterMissing:
+                    pass
 
-            disabled_by: Optional[str] = None
+            disabled_by: str | None = None
             if not entity.entity_registry_enabled_default:
                 disabled_by = DISABLED_INTEGRATION
 
@@ -449,10 +524,10 @@ class EntityPlatform:
         ):
             # If entity already registered, convert entity id to suggestion
             suggested_object_id = split_entity_id(entity.entity_id)[1]
-            entity.entity_id = None
+            generate_new_entity_id = True
 
         # Generate entity ID
-        if entity.entity_id is None:
+        if entity.entity_id is None or generate_new_entity_id:
             suggested_object_id = (
                 suggested_object_id or entity.name or DEVICE_DEFAULT_NAME
             )
@@ -504,7 +579,11 @@ class EntityPlatform:
             # has a chance to finish.
             self.hass.states.async_reserve(entity.entity_id)
 
-        entity.async_on_remove(lambda: self.entities.pop(entity_id))
+        def remove_entity_cb() -> None:
+            """Remove entity from entities list."""
+            self.entities.pop(entity_id)
+
+        entity.async_on_remove(remove_entity_cb)
 
         await entity.add_to_platform_finish()
 
@@ -513,9 +592,7 @@ class EntityPlatform:
 
         This method must be run in the event loop.
         """
-        if self._async_cancel_retry_setup is not None:
-            self._async_cancel_retry_setup()
-            self._async_cancel_retry_setup = None
+        self.async_cancel_retry_setup()
 
         if not self.entities:
             return
@@ -524,10 +601,15 @@ class EntityPlatform:
 
         await asyncio.gather(*tasks)
 
+        self.async_unsub_polling()
+        self._setup_complete = False
+
+    @callback
+    def async_unsub_polling(self) -> None:
+        """Stop polling."""
         if self._async_unsub_polling is not None:
             self._async_unsub_polling()
             self._async_unsub_polling = None
-        self._setup_complete = False
 
     async def async_destroy(self) -> None:
         """Destroy an entity platform.
@@ -550,7 +632,7 @@ class EntityPlatform:
 
     async def async_extract_from_service(
         self, service_call: ServiceCall, expand_group: bool = True
-    ) -> List[Entity]:
+    ) -> list[Entity]:
         """Extract all known and available entities from a service call.
 
         Will return an empty list if entities specified but unknown.
@@ -562,7 +644,13 @@ class EntityPlatform:
         )
 
     @callback
-    def async_register_entity_service(self, name, schema, func, required_features=None):  # type: ignore[no-untyped-def]
+    def async_register_entity_service(
+        self,
+        name: str,
+        schema: dict | vol.Schema,
+        func: str | Callable[..., Any],
+        required_features: Iterable[int] | None = None,
+    ) -> None:
         """Register an entity service.
 
         Services will automatically be shared by all platforms of the same domain.
@@ -621,15 +709,24 @@ class EntityPlatform:
                 await asyncio.gather(*tasks)
 
 
-current_platform: ContextVar[Optional[EntityPlatform]] = ContextVar(
+current_platform: ContextVar[EntityPlatform | None] = ContextVar(
     "current_platform", default=None
 )
 
 
+@callback
+def async_get_current_platform() -> EntityPlatform:
+    """Get the current platform from context."""
+    platform = current_platform.get()
+    if platform is None:
+        raise RuntimeError("Cannot get non-set current platform")
+    return platform
+
+
 @callback
 def async_get_platforms(
-    hass: HomeAssistantType, integration_name: str
-) -> List[EntityPlatform]:
+    hass: HomeAssistant, integration_name: str
+) -> list[EntityPlatform]:
     """Find existing platforms."""
     if (
         DATA_ENTITY_PLATFORM not in hass.data
@@ -637,6 +734,6 @@ def async_get_platforms(
     ):
         return []
 
-    platforms: List[EntityPlatform] = hass.data[DATA_ENTITY_PLATFORM][integration_name]
+    platforms: list[EntityPlatform] = hass.data[DATA_ENTITY_PLATFORM][integration_name]
 
     return platforms
